#ifndef BF609_CHIP_PIN_H
#define BF609_CHIP_PIN_H


typedef unsigned t_chip_pin_id;

#define CHIP_PIN_ID(port, pin, func)   (((port&0xFF)<<16) | ((pin&0xFF) << 8) | (func&0xFF))

/* Извлечение полей ID (internal use) */
#define CHIP_PIN_ID_PORT(id) (((id) >> 16) & 0xFF)
#define CHIP_PIN_ID_PIN(id)  (((id) >> 8) & 0xFF)
#define CHIP_PIN_ID_FINC(id) ((id) & 0xFF)

static const uint32_t chip_fer_addrs[] = {REG_PORTA_FER, REG_PORTB_FER,
                                         REG_PORTC_FER, REG_PORTD_FER,
                                         REG_PORTE_FER, REG_PORTF_FER,
                                         REG_PORTG_FER};

#define CHIP_REG_FER_ADDR(id)   chip_fer_addrs[CHIP_PIN_ID_PORT(id)]
#define CHIP_PIN_REG(reg, id)   *((volatile uint32_t *)(CHIP_REG_FER_ADDR(id) + REG_PORTA_##reg - REG_PORTA_FER))
#define CHIP_PIN_BITMSK(id)     (1UL << CHIP_PIN_ID_PIN(id))

#define CHIP_PIN_GPIO_FUNC      0xFF

/* конфигурация функции пина в соответствии с id */
#define CHIP_PIN_CONFIG(id) do { \
        if (CHIP_PIN_ID_FINC(id) != CHIP_PIN_GPIO_FUNC) { \
            CHIP_PIN_REG(MUX,id) = (CHIP_PIN_REG(MUX,id) & ~(0x3UL << 2*CHIP_PIN_ID_PIN(id))) | \
                               (CHIP_PIN_ID_FINC(id) << 2*CHIP_PIN_ID_PIN(id));\
            CHIP_PIN_REG(FER_SET,id) = CHIP_PIN_BITMSK(id); \
        } else {\
            CHIP_PIN_REG(FER_CLR,id) = CHIP_PIN_BITMSK(id); \
        } \
    } while (0) \


/* перевод пина в конфигурации по-умолчанию (нет спец. функции, вход, запрещен input driver */
#define CHIP_PIN_CONFIG_DEFAULT(id) do { \
        CHIP_PIN_CONFIG(CHIP_PIN_ID(CHIP_PIN_ID_PORT(id), CHIP_PIN_ID_PIN(id), CHIP_PIN_GPIO_FUNC)); \
        CHIP_PIN_REG(DIR_CLR,id) = CHIP_PIN_BITMSK(id); \
        CHIP_PIN_REG(INEN_CLR,id) = CHIP_PIN_BITMSK(id); \
    } while (0)

/* ------------------ Управление GPIO -----------------------*/
    /* управление направлением */
#define CHIP_PIN_DIR_GET(id)         (!!(CHIP_PIN_REG(DIR,id) & CHIP_PIN_BITMSK(id)))
#define CHIP_PIN_DIR_IN(id)          do { \
                                        CHIP_PIN_REG(DIR_CLR,id) = CHIP_PIN_BITMSK(id); \
                                        CHIP_PIN_REG(INEN_SET,id) = CHIP_PIN_BITMSK(id); \
                                     } while(0)

#define CHIP_PIN_DIR_OUT(id)         do { \
                                        CHIP_PIN_REG(DIR_SET,id) = CHIP_PIN_BITMSK(id); \
                                        CHIP_PIN_REG(INEN_CLR,id) = CHIP_PIN_BITMSK(id); \
                                     } while(0)

/* установка уровня на ножке (0 или 1) */
#define CHIP_PIN_OUT(id, val)       if (val) {\
                                        CHIP_PIN_REG(DATA_SET,id) = CHIP_PIN_BITMSK(id); \
                                    } else { \
                                        CHIP_PIN_REG(DATA_CLR,id) = CHIP_PIN_BITMSK(id); \
                                    }

/* чтение уровня на ножке (0 или 1) */
#define CHIP_PIN_IN(id)             (!!(CHIP_PIN_REG(DATA,id) & CHIP_PIN_BITMSK(id)))
/* атомарное инвертирование уровня на ножке */
#define CHIP_PIN_TOGGLE(id)          (CHIP_PIN_REG(DATA_TGL,id) = CHIP_PIN_BITMSK(id))

/* вспомогательный макрос, который конфигурирует порт на вывод с установленным
   начальным значением */
#define CHIP_PIN_CONFIG_OUT(id, val) do { \
                                        CHIP_PIN_OUT(id, val); \
                                        CHIP_PIN_DIR_OUT(id); \
                                        CHIP_PIN_CONFIG(id); \
                                    } while(0)
/* вспомогательный макрос для настройки пина и конфигурирования его на вход */
#define CHIP_PIN_CONFIG_IN(id) do { \
                                    CHIP_PIN_DIR_IN(id); \
                                    CHIP_PIN_CONFIG(id); \
                                } while(0)


/* ------------------ Функции пинов -------------------------------------*/
#define CHIP_PORT_A   0
#define CHIP_PORT_B   1
#define CHIP_PORT_C   2
#define CHIP_PORT_D   3
#define CHIP_PORT_E   4
#define CHIP_PORT_F   5
#define CHIP_PORT_G   6




/********************************** PORT A ************************************/
#define CHIP_PIN_PA00_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA00_SMC0_A03      CHIP_PIN_ID(CHIP_PORT_A, 0, 0)
#define CHIP_PIN_PA00_PPI2_D00      CHIP_PIN_ID(CHIP_PORT_A, 0, 1)
#define CHIP_PIN_PA00_LP0_D0        CHIP_PIN_ID(CHIP_PORT_A, 0, 2)

#define CHIP_PIN_PA01_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA01_SMC0_A04      CHIP_PIN_ID(CHIP_PORT_A, 1, 0)
#define CHIP_PIN_PA01_PPI2_D01      CHIP_PIN_ID(CHIP_PORT_A, 1, 1)
#define CHIP_PIN_PA01_LP0_D1        CHIP_PIN_ID(CHIP_PORT_A, 1, 2)

#define CHIP_PIN_PA02_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA02_SMC0_A05      CHIP_PIN_ID(CHIP_PORT_A, 2, 0)
#define CHIP_PIN_PA02_PPI2_D02      CHIP_PIN_ID(CHIP_PORT_A, 2, 1)
#define CHIP_PIN_PA02_LP0_D2        CHIP_PIN_ID(CHIP_PORT_A, 2, 2)

#define CHIP_PIN_PA03_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA03_SMC0_A06      CHIP_PIN_ID(CHIP_PORT_A, 3, 0)
#define CHIP_PIN_PA03_PPI2_D03      CHIP_PIN_ID(CHIP_PORT_A, 3, 1)
#define CHIP_PIN_PA03_LP0_D3        CHIP_PIN_ID(CHIP_PORT_A, 3, 2)

#define CHIP_PIN_PA04_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA04_SMC0_A07      CHIP_PIN_ID(CHIP_PORT_A, 4, 0)
#define CHIP_PIN_PA04_PPI2_D04      CHIP_PIN_ID(CHIP_PORT_A, 4, 1)
#define CHIP_PIN_PA04_LP0_D4        CHIP_PIN_ID(CHIP_PORT_A, 4, 2)

#define CHIP_PIN_PA05_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA05_SMC0_A08      CHIP_PIN_ID(CHIP_PORT_A, 5, 0)
#define CHIP_PIN_PA05_PPI2_D05      CHIP_PIN_ID(CHIP_PORT_A, 5, 1)
#define CHIP_PIN_PA05_LP0_D5        CHIP_PIN_ID(CHIP_PORT_A, 5, 2)

#define CHIP_PIN_PA06_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA06_SMC0_A09      CHIP_PIN_ID(CHIP_PORT_A, 6, 0)
#define CHIP_PIN_PA06_PPI2_D06      CHIP_PIN_ID(CHIP_PORT_A, 6, 1)
#define CHIP_PIN_PA06_LP0_D6        CHIP_PIN_ID(CHIP_PORT_A, 6, 2)

#define CHIP_PIN_PA07_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA07_SMC0_A10      CHIP_PIN_ID(CHIP_PORT_A, 7, 0)
#define CHIP_PIN_PA07_PPI2_D07      CHIP_PIN_ID(CHIP_PORT_A, 7, 1)
#define CHIP_PIN_PA07_LP0_D7        CHIP_PIN_ID(CHIP_PORT_A, 7, 2)

#define CHIP_PIN_PA08_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA08_SMC0_A11      CHIP_PIN_ID(CHIP_PORT_A, 8, 0)
#define CHIP_PIN_PA08_PPI2_D08      CHIP_PIN_ID(CHIP_PORT_A, 8, 1)
#define CHIP_PIN_PA08_LP1_D0        CHIP_PIN_ID(CHIP_PORT_A, 8, 2)

#define CHIP_PIN_PA09_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA09_SMC0_A12      CHIP_PIN_ID(CHIP_PORT_A, 9, 0)
#define CHIP_PIN_PA09_PPI2_D09      CHIP_PIN_ID(CHIP_PORT_A, 9, 1)
#define CHIP_PIN_PA09_LP1_D1        CHIP_PIN_ID(CHIP_PORT_A, 9, 2)

#define CHIP_PIN_PA10_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA10_SMC0_A14      CHIP_PIN_ID(CHIP_PORT_A, 10, 0)
#define CHIP_PIN_PA10_PPI2_D10      CHIP_PIN_ID(CHIP_PORT_A, 10, 1)
#define CHIP_PIN_PA10_LP1_D2        CHIP_PIN_ID(CHIP_PORT_A, 10, 2)

#define CHIP_PIN_PA11_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA11_SMC0_A15      CHIP_PIN_ID(CHIP_PORT_A, 11, 0)
#define CHIP_PIN_PA11_PPI2_D11      CHIP_PIN_ID(CHIP_PORT_A, 11, 1)
#define CHIP_PIN_PA11_LP1_D3        CHIP_PIN_ID(CHIP_PORT_A, 11, 2)

#define CHIP_PIN_PA12_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA12_SMC0_A17      CHIP_PIN_ID(CHIP_PORT_A, 12, 0)
#define CHIP_PIN_PA12_PPI2_D12      CHIP_PIN_ID(CHIP_PORT_A, 12, 1)
#define CHIP_PIN_PA12_LP1_D4        CHIP_PIN_ID(CHIP_PORT_A, 12, 2)

#define CHIP_PIN_PA13_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA13_SMC0_A18      CHIP_PIN_ID(CHIP_PORT_A, 13, 0)
#define CHIP_PIN_PA13_PPI2_D13      CHIP_PIN_ID(CHIP_PORT_A, 13, 1)
#define CHIP_PIN_PA13_LP1_D5        CHIP_PIN_ID(CHIP_PORT_A, 13, 2)

#define CHIP_PIN_PA14_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA14_SMC0_A19      CHIP_PIN_ID(CHIP_PORT_A, 14, 0)
#define CHIP_PIN_PA14_PPI2_D14      CHIP_PIN_ID(CHIP_PORT_A, 14, 1)
#define CHIP_PIN_PA14_LP1_D6        CHIP_PIN_ID(CHIP_PORT_A, 14, 2)

#define CHIP_PIN_PA15_GPIO          CHIP_PIN_ID(CHIP_PORT_A, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PA15_SMC0_A20      CHIP_PIN_ID(CHIP_PORT_A, 15, 0)
#define CHIP_PIN_PA15_PPI2_D15      CHIP_PIN_ID(CHIP_PORT_A, 15, 1)
#define CHIP_PIN_PA15_LP1_D7        CHIP_PIN_ID(CHIP_PORT_A, 15, 2)

/*********************************** PORT B ***********************************/
#define CHIP_PIN_PB00_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB00_SMC0_NORCLK   CHIP_PIN_ID(CHIP_PORT_B, 0, 0)
#define CHIP_PIN_PB00_PPI2_CLK      CHIP_PIN_ID(CHIP_PORT_B, 0, 1)
#define CHIP_PIN_PB00_LP0_CLK       CHIP_PIN_ID(CHIP_PORT_B, 0, 2)

#define CHIP_PIN_PB01_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB01_SMC0_AMS1     CHIP_PIN_ID(CHIP_PORT_B, 1, 0)
#define CHIP_PIN_PB01_PPI2_FS1      CHIP_PIN_ID(CHIP_PORT_B, 1, 1)
#define CHIP_PIN_PB01_LP0_ACK       CHIP_PIN_ID(CHIP_PORT_B, 1, 2)

#define CHIP_PIN_PB02_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB02_SMC0_A13      CHIP_PIN_ID(CHIP_PORT_B, 2, 0)
#define CHIP_PIN_PB02_PPI2_FS2      CHIP_PIN_ID(CHIP_PORT_B, 2, 1)
#define CHIP_PIN_PB02_LP1_ACK       CHIP_PIN_ID(CHIP_PORT_B, 2, 2)

#define CHIP_PIN_PB03_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB03_SMC0_A16      CHIP_PIN_ID(CHIP_PORT_B, 3, 0)
#define CHIP_PIN_PB03_PPI2_FS3      CHIP_PIN_ID(CHIP_PORT_B, 3, 1)
#define CHIP_PIN_PB03_LP1_CLK       CHIP_PIN_ID(CHIP_PORT_B, 3, 2)

#define CHIP_PIN_PB04_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB04_SMC0_AMS2     CHIP_PIN_ID(CHIP_PORT_B, 4, 0)
#define CHIP_PIN_PB04_SMC0_ABE0     CHIP_PIN_ID(CHIP_PORT_B, 4, 1)
#define CHIP_PIN_PB04_SPT0_AFS      CHIP_PIN_ID(CHIP_PORT_B, 4, 2)

#define CHIP_PIN_PB05_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB05_SMC0_AMS3     CHIP_PIN_ID(CHIP_PORT_B, 5, 0)
#define CHIP_PIN_PB05_SMC0_ABE1     CHIP_PIN_ID(CHIP_PORT_B, 5, 1)
#define CHIP_PIN_PB05_SPT0_ACLK     CHIP_PIN_ID(CHIP_PORT_B, 5, 2)

#define CHIP_PIN_PB06_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB06_SMC0_A21      CHIP_PIN_ID(CHIP_PORT_B, 6, 0)
#define CHIP_PIN_PB06_SPT0_ATDV     CHIP_PIN_ID(CHIP_PORT_B, 6, 1)

#define CHIP_PIN_PB07_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB07_SMC0_A22      CHIP_PIN_ID(CHIP_PORT_B, 7, 0)
#define CHIP_PIN_PB07_PPI2_D16      CHIP_PIN_ID(CHIP_PORT_B, 7, 1)
#define CHIP_PIN_PB07_SPT0_BFS      CHIP_PIN_ID(CHIP_PORT_B, 7, 2)

#define CHIP_PIN_PB08_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB08_SMC0_A23      CHIP_PIN_ID(CHIP_PORT_B, 8, 0)
#define CHIP_PIN_PB08_PPI2_D17      CHIP_PIN_ID(CHIP_PORT_B, 8, 1)
#define CHIP_PIN_PB08_SPT0_BCLK     CHIP_PIN_ID(CHIP_PORT_B, 8, 2)

#define CHIP_PIN_PB09_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB09_SMC0_BGH      CHIP_PIN_ID(CHIP_PORT_B, 9, 0)
#define CHIP_PIN_PB09_SPT0_AD0      CHIP_PIN_ID(CHIP_PORT_B, 9, 2)

#define CHIP_PIN_PB10_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB10_SMC0_A24      CHIP_PIN_ID(CHIP_PORT_B, 10, 0)
#define CHIP_PIN_PB10_SPT0_BD1      CHIP_PIN_ID(CHIP_PORT_B, 10, 2)

#define CHIP_PIN_PB11_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB11_SMC0_A25      CHIP_PIN_ID(CHIP_PORT_B, 11, 0)
#define CHIP_PIN_PB11_SPT0_BD0      CHIP_PIN_ID(CHIP_PORT_B, 11, 2)

#define CHIP_PIN_PB12_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB12_SMC0_BG       CHIP_PIN_ID(CHIP_PORT_B, 12, 0)
#define CHIP_PIN_PB12_SPT0_BTDV     CHIP_PIN_ID(CHIP_PORT_B, 12, 1)
#define CHIP_PIN_PB12_SPT0_AD1      CHIP_PIN_ID(CHIP_PORT_B, 12, 2)

#define CHIP_PIN_PB13_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB13_ETH0_TXEN     CHIP_PIN_ID(CHIP_PORT_B, 13, 0)
#define CHIP_PIN_PB13_PPI1_FS1      CHIP_PIN_ID(CHIP_PORT_B, 13, 1)

#define CHIP_PIN_PB14_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB14_ETH0_REFCLK   CHIP_PIN_ID(CHIP_PORT_B, 14, 0)
#define CHIP_PIN_PB14_PPI1_CLK      CHIP_PIN_ID(CHIP_PORT_B, 14, 1)

#define CHIP_PIN_PB15_GPIO          CHIP_PIN_ID(CHIP_PORT_B, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PB15_ETH0_PTPPPS   CHIP_PIN_ID(CHIP_PORT_B, 15, 0)
#define CHIP_PIN_PB15_PPI1_FS3      CHIP_PIN_ID(CHIP_PORT_B, 15, 1)


/*********************************** PORT C ***********************************/
#define CHIP_PIN_PC00_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC00_ETH0_RXD0     CHIP_PIN_ID(CHIP_PORT_C, 0, 0)
#define CHIP_PIN_PC00_PPI1_D00      CHIP_PIN_ID(CHIP_PORT_C, 0, 1)

#define CHIP_PIN_PC01_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC01_ETH0_RXD1     CHIP_PIN_ID(CHIP_PORT_C, 1, 0)
#define CHIP_PIN_PC01_PPI1_D01      CHIP_PIN_ID(CHIP_PORT_C, 1, 1)

#define CHIP_PIN_PC02_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC02_ETH0_TXD0     CHIP_PIN_ID(CHIP_PORT_C, 2, 0)
#define CHIP_PIN_PC02_PPI1_D02      CHIP_PIN_ID(CHIP_PORT_C, 2, 1)

#define CHIP_PIN_PC03_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC03_ETH0_TXD1     CHIP_PIN_ID(CHIP_PORT_C, 3, 0)
#define CHIP_PIN_PC03_PPI1_D03      CHIP_PIN_ID(CHIP_PORT_C, 3, 1)

#define CHIP_PIN_PC04_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC04_PPI1_D04      CHIP_PIN_ID(CHIP_PORT_C, 4, 1)

#define CHIP_PIN_PC05_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC05_ETH0_CRS      CHIP_PIN_ID(CHIP_PORT_C, 5, 0)
#define CHIP_PIN_PC05_PPI1_D05      CHIP_PIN_ID(CHIP_PORT_C, 5, 1)

#define CHIP_PIN_PC06_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC06_ETH0_MDC      CHIP_PIN_ID(CHIP_PORT_C, 6, 0)
#define CHIP_PIN_PC06_PPI1_D06      CHIP_PIN_ID(CHIP_PORT_C, 6, 1)

#define CHIP_PIN_PC07_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC07_ETH0_MDIO     CHIP_PIN_ID(CHIP_PORT_C, 7, 0)
#define CHIP_PIN_PC07_PPI1_D07      CHIP_PIN_ID(CHIP_PORT_C, 7, 1)

#define CHIP_PIN_PC08_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC08_PPI1_D08      CHIP_PIN_ID(CHIP_PORT_C, 8, 1)

#define CHIP_PIN_PC09_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC09_ETH1_PTPPPS   CHIP_PIN_ID(CHIP_PORT_C, 9, 0)
#define CHIP_PIN_PC09_PPI1_D09      CHIP_PIN_ID(CHIP_PORT_C, 9, 1)

#define CHIP_PIN_PC10_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC10_PPI1_D10      CHIP_PIN_ID(CHIP_PORT_C, 10, 1)

#define CHIP_PIN_PC11_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC11_PPI1_D11      CHIP_PIN_ID(CHIP_PORT_C, 11, 1)

#define CHIP_PIN_PC12_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC12_SPI0_SEL7     CHIP_PIN_ID(CHIP_PORT_C, 12, 0)
#define CHIP_PIN_PC12_PPI1_D12      CHIP_PIN_ID(CHIP_PORT_C, 12, 1)

#define CHIP_PIN_PC13_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC13_SPI0_SEL6     CHIP_PIN_ID(CHIP_PORT_C, 13, 0)
#define CHIP_PIN_PC13_PPI1_D13      CHIP_PIN_ID(CHIP_PORT_C, 13, 1)

#define CHIP_PIN_PC14_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC14_SPI1_SEL7     CHIP_PIN_ID(CHIP_PORT_C, 14, 0)
#define CHIP_PIN_PC14_PPI1_D14      CHIP_PIN_ID(CHIP_PORT_C, 14, 1)

#define CHIP_PIN_PC15_GPIO          CHIP_PIN_ID(CHIP_PORT_C, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PC15_SPI0_SEL4     CHIP_PIN_ID(CHIP_PORT_C, 15, 0)
#define CHIP_PIN_PC15_PPI1_D15      CHIP_PIN_ID(CHIP_PORT_C, 15, 1)


/*********************************** PORT D ***********************************/
#define CHIP_PIN_PD00_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD00_SPI0_D2       CHIP_PIN_ID(CHIP_PORT_D, 0, 0)
#define CHIP_PIN_PD00_PPI1_D16      CHIP_PIN_ID(CHIP_PORT_D, 0, 1)
#define CHIP_PIN_PD00_SPI0_SEL3     CHIP_PIN_ID(CHIP_PORT_D, 0, 2)

#define CHIP_PIN_PD01_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD01_SPI0_D3       CHIP_PIN_ID(CHIP_PORT_D, 1, 0)
#define CHIP_PIN_PD01_PPI1_D17      CHIP_PIN_ID(CHIP_PORT_D, 1, 1)
#define CHIP_PIN_PD01_SPI0_SEL2     CHIP_PIN_ID(CHIP_PORT_D, 1, 2)

#define CHIP_PIN_PD02_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD02_SPI0_MISO     CHIP_PIN_ID(CHIP_PORT_D, 2, 0)

#define CHIP_PIN_PD03_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD03_SPI0_MOSI     CHIP_PIN_ID(CHIP_PORT_D, 3, 0)

#define CHIP_PIN_PD04_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD04_SPI0_CLK      CHIP_PIN_ID(CHIP_PORT_D, 4, 0)

#define CHIP_PIN_PD05_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD05_SPI1_CLK      CHIP_PIN_ID(CHIP_PORT_D, 5, 0)

#define CHIP_PIN_PD06_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD06_PPI1_FS2      CHIP_PIN_ID(CHIP_PORT_D, 6, 1)

#define CHIP_PIN_PD07_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD07_UART0_TX      CHIP_PIN_ID(CHIP_PORT_D, 7, 1)

#define CHIP_PIN_PD08_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD08_UART0_RX      CHIP_PIN_ID(CHIP_PORT_D, 8, 1)

#define CHIP_PIN_PD09_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD09_SPI0_SEL5     CHIP_PIN_ID(CHIP_PORT_D, 9, 0)
#define CHIP_PIN_PD09_UART0_RTS     CHIP_PIN_ID(CHIP_PORT_D, 9, 1)
#define CHIP_PIN_PD09_SPI1_SEL4     CHIP_PIN_ID(CHIP_PORT_D, 9, 2)

#define CHIP_PIN_PD10_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD10_SPI0_RDY      CHIP_PIN_ID(CHIP_PORT_D, 10, 0)
#define CHIP_PIN_PD10_UART0_CTS     CHIP_PIN_ID(CHIP_PORT_D, 10, 1)
#define CHIP_PIN_PD10_SPI1_SEL3     CHIP_PIN_ID(CHIP_PORT_D, 10, 2)

#define CHIP_PIN_PD11_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD11_SPI0_SEL1     CHIP_PIN_ID(CHIP_PORT_D, 11, 0)

#define CHIP_PIN_PD12_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD12_SPI1_SEL1     CHIP_PIN_ID(CHIP_PORT_D, 12, 0)
#define CHIP_PIN_PD12_PPI0_D20      CHIP_PIN_ID(CHIP_PORT_D, 12, 1)
#define CHIP_PIN_PD12_SPT1_AD1      CHIP_PIN_ID(CHIP_PORT_D, 12, 2)

#define CHIP_PIN_PD13_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD13_SPI1_MOSI     CHIP_PIN_ID(CHIP_PORT_D, 13, 0)

#define CHIP_PIN_PD14_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD14_SPI1_MISO     CHIP_PIN_ID(CHIP_PORT_D, 14, 0)

#define CHIP_PIN_PD15_GPIO          CHIP_PIN_ID(CHIP_PORT_D, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PD15_SPI1_SEL2     CHIP_PIN_ID(CHIP_PORT_D, 15, 0)
#define CHIP_PIN_PD15_PPI0_D21      CHIP_PIN_ID(CHIP_PORT_D, 15, 1)
#define CHIP_PIN_PD15_SPT1_AD0      CHIP_PIN_ID(CHIP_PORT_D, 15, 2)

/*********************************** PORT E ***********************************/
#define CHIP_PIN_PE00_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE00_SPI1_D3       CHIP_PIN_ID(CHIP_PORT_E, 0, 0)
#define CHIP_PIN_PE00_PPI0_D18      CHIP_PIN_ID(CHIP_PORT_E, 0, 1)
#define CHIP_PIN_PE00_SPT1_BD1      CHIP_PIN_ID(CHIP_PORT_E, 0, 2)

#define CHIP_PIN_PE01_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE01_SPI1_D2       CHIP_PIN_ID(CHIP_PORT_E, 1, 0)
#define CHIP_PIN_PE01_PPI0_D19      CHIP_PIN_ID(CHIP_PORT_E, 1, 1)
#define CHIP_PIN_PE01_SPT1_BD0      CHIP_PIN_ID(CHIP_PORT_E, 1, 2)

#define CHIP_PIN_PE02_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE02_SPI1_RDY      CHIP_PIN_ID(CHIP_PORT_E, 2, 0)
#define CHIP_PIN_PE02_PPI0_D22      CHIP_PIN_ID(CHIP_PORT_E, 2, 1)
#define CHIP_PIN_PE02_SPT1_ACLK     CHIP_PIN_ID(CHIP_PORT_E, 2, 2)

#define CHIP_PIN_PE03_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE03_PPI0_D16      CHIP_PIN_ID(CHIP_PORT_E, 3, 1)
#define CHIP_PIN_PE03_ACM0_FS       CHIP_PIN_ID(CHIP_PORT_E, 3, 2)
#define CHIP_PIN_PE03_SPT1_BFS      CHIP_PIN_ID(CHIP_PORT_E, 3, 2)

#define CHIP_PIN_PE04_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE04_PPI0_D17      CHIP_PIN_ID(CHIP_PORT_E, 4, 1)
#define CHIP_PIN_PE04_ACM0_CLK      CHIP_PIN_ID(CHIP_PORT_E, 4, 2)
#define CHIP_PIN_PE04_SPT1_BCLK     CHIP_PIN_ID(CHIP_PORT_E, 4, 2)

#define CHIP_PIN_PE05_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE05_PPI0_D23      CHIP_PIN_ID(CHIP_PORT_E, 5, 1)
#define CHIP_PIN_PE05_SPT1_AFS      CHIP_PIN_ID(CHIP_PORT_E, 5, 2)

#define CHIP_PIN_PE06_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE06_SPT1_ATDV     CHIP_PIN_ID(CHIP_PORT_E, 6, 0)
#define CHIP_PIN_PE06_PPI0_FS3      CHIP_PIN_ID(CHIP_PORT_E, 6, 1)
#define CHIP_PIN_PE06_LP3_CLK       CHIP_PIN_ID(CHIP_PORT_E, 6, 2)

#define CHIP_PIN_PE07_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE07_SPT1_BTDV     CHIP_PIN_ID(CHIP_PORT_E, 7, 0)
#define CHIP_PIN_PE07_PPI0_FS2      CHIP_PIN_ID(CHIP_PORT_E, 7, 1)
#define CHIP_PIN_PE07_LP3_ACK       CHIP_PIN_ID(CHIP_PORT_E, 7, 2)

#define CHIP_PIN_PE08_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE08_PWM0_SYNC     CHIP_PIN_ID(CHIP_PORT_E, 8, 0)
#define CHIP_PIN_PE08_PPI0_FS1      CHIP_PIN_ID(CHIP_PORT_E, 8, 1)
#define CHIP_PIN_PE08_LP2_ACK       CHIP_PIN_ID(CHIP_PORT_E, 8, 2)

#define CHIP_PIN_PE09_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE09_PPI0_CLK      CHIP_PIN_ID(CHIP_PORT_E, 9, 1)
#define CHIP_PIN_PE09_LP2_CLK       CHIP_PIN_ID(CHIP_PORT_E, 9, 2)

#define CHIP_PIN_PE10_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE10_ETH1_MDC      CHIP_PIN_ID(CHIP_PORT_E, 10, 0)
#define CHIP_PIN_PE10_PWM1_DL       CHIP_PIN_ID(CHIP_PORT_E, 10, 1)
#define CHIP_PIN_PE10_RSI0_D6       CHIP_PIN_ID(CHIP_PORT_E, 10, 2)

#define CHIP_PIN_PE11_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE11_ETH1_MDIO     CHIP_PIN_ID(CHIP_PORT_E, 11, 0)
#define CHIP_PIN_PE11_PWM1_DH       CHIP_PIN_ID(CHIP_PORT_E, 11, 1)
#define CHIP_PIN_PE11_RSI0_D7       CHIP_PIN_ID(CHIP_PORT_E, 11, 2)

#define CHIP_PIN_PE12_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE12_PWM1_CL       CHIP_PIN_ID(CHIP_PORT_E, 12, 1)
#define CHIP_PIN_PE12_RSI0_D5       CHIP_PIN_ID(CHIP_PORT_E, 12, 2)

#define CHIP_PIN_PE13_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE13_ETH1_CRS      CHIP_PIN_ID(CHIP_PORT_E, 13, 0)
#define CHIP_PIN_PE13_PWM1_CH       CHIP_PIN_ID(CHIP_PORT_E, 13, 1)
#define CHIP_PIN_PE13_RSI0_D4       CHIP_PIN_ID(CHIP_PORT_E, 13, 2)

#define CHIP_PIN_PE14_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE14_SPT2_ATDV     CHIP_PIN_ID(CHIP_PORT_E, 14, 1)
#define CHIP_PIN_PE14_TM0_TMR0      CHIP_PIN_ID(CHIP_PORT_E, 14, 2)

#define CHIP_PIN_PE15_GPIO          CHIP_PIN_ID(CHIP_PORT_E, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PE15_ETH1_RXD1     CHIP_PIN_ID(CHIP_PORT_E, 15, 0)
#define CHIP_PIN_PE15_PWM1_BL       CHIP_PIN_ID(CHIP_PORT_E, 15, 1)
#define CHIP_PIN_PE15_RSI0_D3       CHIP_PIN_ID(CHIP_PORT_E, 15, 2)


/*********************************** PORT F ***********************************/
#define CHIP_PIN_PF00_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF00_PWM0_AL       CHIP_PIN_ID(CHIP_PORT_F, 0, 0)
#define CHIP_PIN_PF00_PPI0_D00      CHIP_PIN_ID(CHIP_PORT_F, 0, 1)
#define CHIP_PIN_PF00_LP2_D0        CHIP_PIN_ID(CHIP_PORT_F, 0, 2)

#define CHIP_PIN_PF01_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF01_PWM0_AH       CHIP_PIN_ID(CHIP_PORT_F, 1, 0)
#define CHIP_PIN_PF01_PPI0_D01      CHIP_PIN_ID(CHIP_PORT_F, 1, 1)
#define CHIP_PIN_PF01_LP2_D1        CHIP_PIN_ID(CHIP_PORT_F, 1, 2)

#define CHIP_PIN_PF02_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF02_PWM0_BL       CHIP_PIN_ID(CHIP_PORT_F, 2, 0)
#define CHIP_PIN_PF02_PPI0_D02      CHIP_PIN_ID(CHIP_PORT_F, 2, 1)
#define CHIP_PIN_PF02_LP2_D2        CHIP_PIN_ID(CHIP_PORT_F, 2, 2)

#define CHIP_PIN_PF03_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF03_PWM0_BH       CHIP_PIN_ID(CHIP_PORT_F, 3, 0)
#define CHIP_PIN_PF03_PPI0_D03      CHIP_PIN_ID(CHIP_PORT_F, 3, 1)
#define CHIP_PIN_PF03_LP2_D3        CHIP_PIN_ID(CHIP_PORT_F, 3, 2)

#define CHIP_PIN_PF04_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF04_PWM0_CL       CHIP_PIN_ID(CHIP_PORT_F, 4, 0)
#define CHIP_PIN_PF04_PPI0_D04      CHIP_PIN_ID(CHIP_PORT_F, 4, 1)
#define CHIP_PIN_PF04_LP2_D4        CHIP_PIN_ID(CHIP_PORT_F, 4, 2)

#define CHIP_PIN_PF05_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF05_PWM0_CH       CHIP_PIN_ID(CHIP_PORT_F, 5, 0)
#define CHIP_PIN_PF05_PPI0_D05      CHIP_PIN_ID(CHIP_PORT_F, 5, 1)
#define CHIP_PIN_PF05_LP2_D5        CHIP_PIN_ID(CHIP_PORT_F, 5, 2)

#define CHIP_PIN_PF06_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF06_PWM0_DL       CHIP_PIN_ID(CHIP_PORT_F, 6, 0)
#define CHIP_PIN_PF06_PPI0_D06      CHIP_PIN_ID(CHIP_PORT_F, 6, 1)
#define CHIP_PIN_PF06_LP2_D6        CHIP_PIN_ID(CHIP_PORT_F, 6, 2)

#define CHIP_PIN_PF07_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF07_PWM0_DH       CHIP_PIN_ID(CHIP_PORT_F, 7, 0)
#define CHIP_PIN_PF07_PPI0_D07      CHIP_PIN_ID(CHIP_PORT_F, 7, 1)
#define CHIP_PIN_PF07_LP2_D7        CHIP_PIN_ID(CHIP_PORT_F, 7, 2)

#define CHIP_PIN_PF08_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF08_SPI1_SEL5     CHIP_PIN_ID(CHIP_PORT_F, 8, 0)
#define CHIP_PIN_PF08_PPI0_D08      CHIP_PIN_ID(CHIP_PORT_F, 8, 1)
#define CHIP_PIN_PF08_LP3_D0        CHIP_PIN_ID(CHIP_PORT_F, 8, 2)

#define CHIP_PIN_PF09_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF09_SPI1_SEL6     CHIP_PIN_ID(CHIP_PORT_F, 9, 0)
#define CHIP_PIN_PF09_PPI0_D09      CHIP_PIN_ID(CHIP_PORT_F, 9, 1)
#define CHIP_PIN_PF09_LP3_D1        CHIP_PIN_ID(CHIP_PORT_F, 9, 2)

#define CHIP_PIN_PF10_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF10_ACM0_A4       CHIP_PIN_ID(CHIP_PORT_F, 10, 0)
#define CHIP_PIN_PF10_PPI0_D10      CHIP_PIN_ID(CHIP_PORT_F, 10, 1)
#define CHIP_PIN_PF10_LP3_D2        CHIP_PIN_ID(CHIP_PORT_F, 10, 2)

#define CHIP_PIN_PF11_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF11_PPI0_D11      CHIP_PIN_ID(CHIP_PORT_F, 11, 1)
#define CHIP_PIN_PF11_LP3_D3        CHIP_PIN_ID(CHIP_PORT_F, 11, 2)

#define CHIP_PIN_PF12_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF12_ACM0_A2       CHIP_PIN_ID(CHIP_PORT_F, 12, 0)
#define CHIP_PIN_PF12_PPI0_D12      CHIP_PIN_ID(CHIP_PORT_F, 12, 1)
#define CHIP_PIN_PF12_LP3_D4        CHIP_PIN_ID(CHIP_PORT_F, 12, 2)

#define CHIP_PIN_PF13_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF13_ACM0_A3       CHIP_PIN_ID(CHIP_PORT_F, 13, 0)
#define CHIP_PIN_PF13_PPI0_D13      CHIP_PIN_ID(CHIP_PORT_F, 13, 1)
#define CHIP_PIN_PF13_LP3_D5        CHIP_PIN_ID(CHIP_PORT_F, 13, 2)

#define CHIP_PIN_PF14_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF14_ACM0_A0       CHIP_PIN_ID(CHIP_PORT_F, 14, 0)
#define CHIP_PIN_PF14_PPI0_D14      CHIP_PIN_ID(CHIP_PORT_F, 14, 1)
#define CHIP_PIN_PF14_LP3_D6        CHIP_PIN_ID(CHIP_PORT_F, 14, 2)

#define CHIP_PIN_PF15_GPIO          CHIP_PIN_ID(CHIP_PORT_F, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PF15_ACM0_A1       CHIP_PIN_ID(CHIP_PORT_F, 15, 0)
#define CHIP_PIN_PF15_PPI0_D15      CHIP_PIN_ID(CHIP_PORT_F, 15, 1)
#define CHIP_PIN_PF15_LP3_D7        CHIP_PIN_ID(CHIP_PORT_F, 15, 2)


/*********************************** PORT F ***********************************/
#define CHIP_PIN_PG00_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 0, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG00_ETH1_RXD0     CHIP_PIN_ID(CHIP_PORT_G, 0, 0)
#define CHIP_PIN_PG00_PWM1_BH       CHIP_PIN_ID(CHIP_PORT_G, 0, 1)
#define CHIP_PIN_PG00_RSI0_D2       CHIP_PIN_ID(CHIP_PORT_G, 0, 2)

#define CHIP_PIN_PG01_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 1, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG01_SPT2_AFS      CHIP_PIN_ID(CHIP_PORT_G, 1, 0)
#define CHIP_PIN_PG01_TM0_TMR2      CHIP_PIN_ID(CHIP_PORT_G, 1, 1)
#define CHIP_PIN_PG01_CAN0_TX       CHIP_PIN_ID(CHIP_PORT_G, 1, 2)

#define CHIP_PIN_PG02_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 2, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG02_ETH1_TXD1     CHIP_PIN_ID(CHIP_PORT_G, 2, 0)
#define CHIP_PIN_PG02_PWM1_AL       CHIP_PIN_ID(CHIP_PORT_G, 2, 1)
#define CHIP_PIN_PG02_RSI0_D1       CHIP_PIN_ID(CHIP_PORT_G, 2, 2)

#define CHIP_PIN_PG03_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 3, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG03_ETH1_TXD0     CHIP_PIN_ID(CHIP_PORT_G, 3, 0)
#define CHIP_PIN_PG03_PWM1_AH       CHIP_PIN_ID(CHIP_PORT_G, 3, 1)
#define CHIP_PIN_PG03_RSI0_D0       CHIP_PIN_ID(CHIP_PORT_G, 3, 2)

#define CHIP_PIN_PG04_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 4, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG04_SPT2_ACLK     CHIP_PIN_ID(CHIP_PORT_G, 4, 0)
#define CHIP_PIN_PG04_TM0_TMR1      CHIP_PIN_ID(CHIP_PORT_G, 4, 1)
#define CHIP_PIN_PG04_CAN0_RX       CHIP_PIN_ID(CHIP_PORT_G, 4, 2)

#define CHIP_PIN_PG05_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 5, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG05_ETH1_TXEN     CHIP_PIN_ID(CHIP_PORT_G, 5, 0)
#define CHIP_PIN_PG05_RSI0_CMD      CHIP_PIN_ID(CHIP_PORT_G, 5, 1)
#define CHIP_PIN_PG05_PWM1_SYNC     CHIP_PIN_ID(CHIP_PORT_G, 5, 2)

#define CHIP_PIN_PG06_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 6, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG06_ETH1_REFCLK   CHIP_PIN_ID(CHIP_PORT_G, 6, 0)
#define CHIP_PIN_PG06_RSI0_CLK      CHIP_PIN_ID(CHIP_PORT_G, 6, 1)
#define CHIP_PIN_PG06_SPT2_BTDV     CHIP_PIN_ID(CHIP_PORT_G, 6, 2)

#define CHIP_PIN_PG07_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 7, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG07_SPT2_BFS      CHIP_PIN_ID(CHIP_PORT_G, 7, 0)
#define CHIP_PIN_PG07_TM0_TMR5      CHIP_PIN_ID(CHIP_PORT_G, 7, 1)

#define CHIP_PIN_PG08_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 8, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG08_SPT2_AD1      CHIP_PIN_ID(CHIP_PORT_G, 8, 0)
#define CHIP_PIN_PG08_TM0_TMR3      CHIP_PIN_ID(CHIP_PORT_G, 8, 1)

#define CHIP_PIN_PG09_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 9, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG09_SPT2_AD0      CHIP_PIN_ID(CHIP_PORT_G, 9, 0)
#define CHIP_PIN_PG09_TM0_TMR4      CHIP_PIN_ID(CHIP_PORT_G, 9, 1)

#define CHIP_PIN_PG10_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 10, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG10_UART1_RTS     CHIP_PIN_ID(CHIP_PORT_G, 10, 0)
#define CHIP_PIN_PG10_SPT2_BCLK     CHIP_PIN_ID(CHIP_PORT_G, 10, 1)

#define CHIP_PIN_PG11_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 11, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG11_SPT2_BD1      CHIP_PIN_ID(CHIP_PORT_G, 11, 0)
#define CHIP_PIN_PG11_TM0_TMR6      CHIP_PIN_ID(CHIP_PORT_G, 11, 1)

#define CHIP_PIN_PG12_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 12, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG12_SPT2_BD0      CHIP_PIN_ID(CHIP_PORT_G, 12, 0)
#define CHIP_PIN_PG12_TM0_TMR7      CHIP_PIN_ID(CHIP_PORT_G, 12, 1)

#define CHIP_PIN_PG13_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 13, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG13_UART1_CTS     CHIP_PIN_ID(CHIP_PORT_G, 13, 0)

#define CHIP_PIN_PG14_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 14, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG14_UART1_RX      CHIP_PIN_ID(CHIP_PORT_G, 14, 0)
#define CHIP_PIN_PG14_SYS_IDLE1     CHIP_PIN_ID(CHIP_PORT_G, 14, 1)

#define CHIP_PIN_PG15_GPIO          CHIP_PIN_ID(CHIP_PORT_G, 15, CHIP_PIN_GPIO_FUNC)
#define CHIP_PIN_PG15_UART1_TX      CHIP_PIN_ID(CHIP_PORT_G, 15, 0)
#define CHIP_PIN_PG15_SYS_IDLE0     CHIP_PIN_ID(CHIP_PORT_G, 15, 1)
#define CHIP_PIN_PG15_SYS_SLEEP     CHIP_PIN_ID(CHIP_PORT_G, 15, 2)

#endif // BF609_CHIP_PIN_H

