#ifndef CHIP_ARM_H
#define CHIP_ARM_H

/*================================================================================================*
 * ---- Атомарный доступ к битам памяти и регистров (bit-banding) ----
 * BIT_BAND(var, bitno) = 0 или 1
 *================================================================================================*/
#define BIT_BAND(var, bitno) \
    (*((volatile uint32_t*)(((uint32_t)(&(var)) & 0xFE000000) + 0x02000000 + \
    (((((uint32_t)(&(var)) & 0xFFFFF) << 3) + (bitno)) << 2))))


/*================================================================================================*
 * ---- Intrinsic функции управления процессором ----
 * (В CMSIS описаны неудачно, без memory clobber. В линуксе - clobber есть.)
 *================================================================================================*/
#define __sev() asm volatile ("sev" ::: "memory")   /* send event (multiproc) */
#define __wfe() asm volatile ("wfe" ::: "memory")   /* wait for event (multiproc) */
#define __wfi() asm volatile ("wfi" ::: "memory")   /* wait for interrupt */
#define __isb() asm volatile ("isb" ::: "memory")   /* instruction sync barrier */
#define __dsb() asm volatile ("dsb" ::: "memory")   /* data sync barrier, e.g. after disable IRQ */
#define __dmb() asm volatile ("dmb" ::: "memory")   /* data memory barrier, e.g. before DMA */

#define __irq_enable()      asm volatile ("cpsie i" ::: "memory", "cc")
#define __irq_disable()     asm volatile ("cpsid i" ::: "memory", "cc")
#define __fault_enable()    asm volatile ("cpsie f" ::: "memory", "cc")
#define __fault_disable()   asm volatile ("cpsid f" ::: "memory", "cc")
#define __irq_save(var) \
    asm volatile ("mrs %[reg], primask" : [reg] "=r" (var) :: "memory", "cc")
#define __irq_restore(var) \
    asm volatile ("msr primask, %[reg]" : : [reg] "r" (var): "memory", "cc")


#endif // CHIP_ARM_H
