#ifndef LPC_CONFIG_H
#define LPC_CONFIG_H


#define LPC_CLKF_OSC        16000000UL             /* Частота кварца OSC, Гц; 0 - не используется */

#define LPC_MAINCLK_SRC     LPC_MAINCLK_SRC_PLLOUT /* источник главного клока */
#define LPC_SYSCLK_DIV      1   /* 1..255.  Fcpu = main clock / LPC_SYSCLK_DIV */

/****              System PLL (Fout = Fin * M, Fcco = Fout*2*P)    ******/
#define LPC_USE_SYSPLL      1               /* Включение системного PLL */
#define LPC_SYSPLL_SRC      LPC_SYSPLL_SRC_SYSOSC
#define LPC_SYSPLL_M        3              /* M = 1..32  */
#define LPC_SYSPLL_P        2              /* P = 1,2,4,8 */

/****                USB              ******/
#define LPC_USE_USBCLK      1
#define LPC_USBCLK_SRC      LPC_USBCLK_SRC_USBPLL
#define LPC_USBCLK_DIV      1
/* Настройки PLL для USB (Fout = Fin * M, Fcco = Fout*2*P).
 * Должны быть действительные даже если используем main clock для USB */
#define LPC_USBPLL_SRC      LPC_USBPLL_SRC_SYSOSC
#define LPC_USBPLL_M        3  /* M = 1..32  */
#define LPC_USBPLL_P        2  /* P = 1,2,4,8 */


/****                WDT OSC          *******/
#define LPC_USE_WDTOSC      0
#define LPC_WDT_FREQSEL     LPC_WDT_FREQSEL_4_60
#define LPC_WDT_DIVSEL      1 /* 0..31.  Fwdtosc = Fana / (2 * (1+LPC_WDT_DIVSEL)) */

/* использование ножки CLKOUT для генерации выходной частоты */
#define LPC_USE_CLKOUT      1
#define LPC_CLKOUT_SRC      LPC_CLKOUT_SRC_MAINCLK
#define LPC_CLKOUT_DIV      4

/* Если не 0, то по завершению инициализации нужно отключить IRC осциллятор */
#define LPC_IRC_PD          1


/* включать ли WDT */
#define LPC_INIT_WDT 1

#define LPC_WDTCLK_SRC          LPC_WDTCLK_SRC_WDTOSC               /* источник клока для WDT */
/* определяет время по которому будут сброс чипа, если не было перезапуска wdt*/
#define LPC_WDT_TIMEOUT_VAL     LPC_WDT_TICKS_MS(LBOOT_WDT_TOUT)
/* за сколько времени до сброса чипа генерировать прерывание (если разрешено) */
#define LPC_WDT_WARNTIME_VAL    LPC_WDT_TICKS_MS(LBOOT_WDT_TOUT)
/* за какое макс. время до истечения таймера можно перезапускать wdt */
#define LPC_WDT_WINDOW_VAL      0x01000000
/* защита от изменения время сброса. но приводит и к тому, что перезпустить
 * wdt можно только когда осталось не более LPC_WDT_WARNTIME_VAL времени до сброса... */
#define LPC_WDT_TC_PROTECT      0
/* защита на изменение или отключение клока WDT */
#define LPC_WDT_CLK_LOCK        0


#endif // LPC_CONFIG_H
