#include "chip.h"
#include "lpc_config.h"

#define PLL_FCCO_MIN   156000000UL
#define PLL_FCCO_MAX   320000000UL


#ifndef LPC_MAINCLK_SRC
    #error "Main clock source is not defined"
#else
    #if (LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_PLLOUT) && !LPC_USE_SYSPLL
        #error "Main clock source is SYSPLL but SYSPLL is not enabled"
    #elif (LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_PLLIN) && !defined LPC_SYSPLL_SRC
        #error "Main clock source is PLLIN but PLL input source is not defined"
    #elif (LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_WDTOSC) && !LPC_USE_WDTOSC
        #error "Main clock source is WDT OSC, but WDT OSC is not enabled"
    #endif
#endif

#if (LPC_SYSCLK_DIV < 1 ) || (LPC_SYSCLK_DIV > 255)
    #error "Invalid value of LPC_SYSCLK_DIV"
#endif

#if LPC_CLKF_OSC
    #if (LPC_CLKF_OSC < 1000000 ) || (LPC_CLKF_OSC > 25000000)
        #error "Invalid value of LPC_CLKF_OSC"
    #endif
#endif

#if LPC_USE_WDTOSC
    #if (LPC_WDT_DIVSEL < 0) || (LPC_WDT_DIVSEL > 31)
        #error "invalid value of LPC_WDT_DIVSEL"
    #endif
#endif

#if LPC_USE_SYSPLL
    #define LPC_CLKF_SYSPLL_FCCO   (LPC_CLKF_SYSPLL_OUT*2*LPC_SYSPLL_P)

    #if (LPC_SYSPLL_M < 1) || (LPC_SYSPLL_M > 32)
        #error "Invalid value of LPC_SYSPLL_M"
    #endif

    #if (LPC_SYSPLL_P != 1) && (LPC_SYSPLL_P != 2) && (LPC_SYSPLL_P != 4) && (LPC_SYSPLL_P != 8)
        #error "Invalid value of LPC_SYSPLL_P"
    #endif

    #if (LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_SYSOSC) && !(LPC_CLKF_OSC)
        #error "PLL source is oscillator, but OSC not enabled!"
    #endif

    #if ((LPC_CLKF_SYSPLL_IN < 10000000) || (LPC_CLKF_SYSPLL_IN > 25000000))
        #error "invalid system PLL input frequency"
    #endif

    #if ((LPC_CLKF_SYSPLL_FCCO < PLL_FCCO_MIN) || (LPC_CLKF_SYSPLL_FCCO > PLL_FCCO_MAX))
        #error "invalid system PLL Fcco frequency"
    #endif

#endif


#if LPC_USE_USBCLK
    #define LPC_CLKF_USBPLL_FCCO   (LPC_CLKF_USBPLL_OUT*2*LPC_USBPLL_P)

    #if (LPC_USBPLL_M < 1) || (LPC_USBPLL_M > 32)
        #error "Invalid value of LPC_USBPLL_M"
    #endif

    #if (LPC_USBPLL_P != 1) && (LPC_USBPLL_P != 2) && (LPC_USBPLL_P != 4) && (LPC_USBPLL_P != 8)
        #error "Invalid value of LPC_USBPLL_P"
    #endif

    #if (LPC_SYSPLL_SRC == LPC_USBPLL_SRC_SYSOSC) && !(LPC_CLKF_OSC)
        #error "USBPLL source is oscillator, but OSC is not enabled!"
    #endif


    #if ((LPC_CLKF_USBPLL_IN < 10000000) || (LPC_CLKF_USBPLL_IN > 25000000))
        #error "Invalid USBPLL input frequency"
    #endif

    #if ((LPC_CLKF_USBPLL_FCCO < PLL_FCCO_MIN) || (LPC_CLKF_USBPLL_FCCO > PLL_FCCO_MAX))
        #error "invalid USB PLL Fcco frequency"
    #endif


    #if (LPC_USBCLK_DIV < 1) || (LPC_USBCLK_DIV > 255)
       #error "Invalid value of LPC_USBCLK_DIV"
    #endif

    #if LPC_CLKF_USB != 48000000
        #error "Invalid USB frequency"
    #endif
#endif


#if LPC_USE_CLKOUT
    #if (LPC_CLKOUT_DIV > 255)
        #error "Invalid value of LPC_CLKOUT_DIV"
    #endif

    #if (LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_SYSOSC) && !(LPC_CLKF_OSC)
        #error "CLKOUT source is oscillator, but OSC is not enabled!"
    #endif

    #if (LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_WDTOSC) && !LPC_USE_WDTOSC
        #error "CLKOUT source is WDT OSC, but WDT OSC is not enabled"
    #endif
#endif

#if LPC_IRC_PD
    #if  LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_IRC && \
            (LPC_USE_SYSPLL || (LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_PLLIN))
        #error "IRC power down configured, but it used for system PLL source"
    #endif
    #if (LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_IRC)
        #error "IRC power down configured, but it used for main clock source"
    #endif
    #if LPC_USE_USBCLK && (LPC_USBPLL_SRC == LPC_USBPLL_SRC_IRC)
        #error "IRC power down configured, but it used for USB PLL source"
    #endif
    #if LPC_USE_CLKOUT && (LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_IRC)
        #error "IRC power down configured, but it used for CLKOUT source"
    #endif
#endif

#if (LPC_USART0_CLK_DIV < 0) || (LPC_USART0_CLK_DIV > 255)
    #error "Invalid value of LPC_USART0_CLK_DIV"
#endif

#if (LPC_SSP0_CLK_DIV < 0) || (LPC_SSP0_CLK_DIV > 255)
    #error "Invalid value of LPC_SSP0_CLK_DIV"
#endif

#if (LPC_SSP1_CLK_DIV < 0) || (LPC_SSP1_CLK_DIV > 255)
    #error "Invalid value of LPC_SSP1_CLK_DIV"
#endif

#if (LPC_FRG_CLK_DIV < 0) || (LPC_FRG_CLK_DIV > 255)
    #error "Invalid value of LPC_FRG_CLK_DIV"
#endif

#if LPC_FRG_CLK_DIV
    #if (LPC_USART_FRG_DIV < 1) || (LPC_USART_FRG_DIV > 256)
        #error "Invalid value of LPC_USART_FRG_DIV"
    #endif

    #if (LPC_USART_FRG_MUL < 0) || (LPC_USART_FRG_MUL > 255)
        #error "Invalid value of LPC_USART_FRG_MUL"
    #endif
#endif


#if LPC_INIT_WDT
    #ifndef LPC_WDT_WINDOW_VAL
        #define LPC_WDT_WINDOW_VAL 0x01000000
    #endif

    #if LPC_WDT_TIMEOUT_VAL > 0x01000000
        #error "WDT timeout value too high"
    #endif
    #if LPC_WDT_TIMEOUT_VAL < 256
        #error "WDT timeout too low"
    #endif
    #if LPC_WDT_WINDOW_VAL > 0x01000000
        #error "WDT window value too high"
    #endif

    #if LPC_WDT_WARNTIME_VAL  > 1024
        #error "WDT warn time too high"
    #endif


#endif




#define PLL_CTL_REG_VAL(m,n)  (((m)-1) | (((n)==2 ? 1 : (n)==4 ? 2 : (n)==8 ? 3 : 0) << 5))



__attribute__ ((__noinline__)) void lpc_delay_clk(unsigned int clk_count) {
#define CYCLES_OVERHEAD     7
    /* Задержка примерно на clk_count тактов, min = 11 тактов */
    asm volatile (
         " .syntax unified\n"
         "  subs %[reg], %[reg], %[adj]\n"
         "  asrs %[reg], %[reg], #2\n"  /* 4 cycles per loop */
         "0:"
         "  subs %[reg], %[reg], #0\n"
         "  subs %[reg], %[reg], #1\n"
         "  bgt 0b\n"
         " .syntax divided"
         : [reg] "+r" (clk_count)
         : [adj] "i" (CYCLES_OVERHEAD - 3)
         : "cc"
        );
#undef CYCLES_OVERHEAD
    }



void chip_init(void) {
    LPC_SYSCTL->PDRUNCFG &= ~(SYSCTL_POWERDOWN_IRC_PD | SYSCTL_POWERDOWN_IRCOUT_PD);

    LPC_SYSCTL->MAINCLKSEL = LPC_MAINCLK_SRC_IRC;
    LPC_SYSCTL->MAINCLKUEN = 0;
    LPC_SYSCTL->MAINCLKUEN = 1;
    LPC_SYSCTL->SYSAHBCLKDIV = 1;

#if LPC_USE_WDTOSC
    LPC_SYSCTL->WDTOSCCTRL = (LPC_WDT_FREQSEL << 5) | (LPC_WDT_DIVSEL & 0x1F);
    LPC_SYSCTL->PDRUNCFG &= ~SYSCTL_POWERDOWN_WDTOSC_PD;
#endif

#if LPC_INIT_WDT
    LPC_CLK_EN(SYSCTL_CLOCK_WDT);
    LPC_WWDT->CLKSEL =  LPC_WDTCLK_SRC;
#if LPC_WDT_CLK_LOCK
    LPC_WWDT->CLKSEL |= WWDT_CLKSEL_LOCK;
#endif

    LPC_WWDT->TC = LPC_WDT_TIMEOUT_VAL - 1;
    LPC_WWDT->MOD |=  WWDT_WDMOD_WDRESET | WWDT_WDMOD_WDINT | WWDT_WDMOD_WDEN;
#if LPC_WDT_CLK_LOCK
    LPC_WWDT->MOD |= WWDT_WDMOD_LOCK;
#endif
    LPC_WWDT->WINDOW = LPC_WDT_WINDOW_VAL - 1;
    LPC_WWDT->WARNINT = LPC_WDT_WARNTIME_VAL - 1;

    LPC_WWDT->FEED = 0xAA;
    LPC_WWDT->FEED = 0x55;

#if LPC_WDT_TC_PROTECT
    lpc_delay_clk(3*LPC_CLKF_WDT/LPC_CLKF_IRC);
    LPC_WWDT->MOD |= WWDT_WDMOD_WDPROTECT;
#endif
#endif



#if LPC_CLKF_OSC
    LPC_PIN_CONFIG(LPC_PIN_PIO2_0_XTALIN,  0);
    LPC_PIN_CONFIG(LPC_PIN_PIO2_1_XTALOUT, 0);

    LPC_SYSCTL->PDRUNCFG &= ~SYSCTL_POWERDOWN_SYSOSC_PD;
    /* необходима задержка 500 мкс после включения питания OSC.
     * учитываем, что вначале всегда частота берется от IRC */
    lpc_delay_clk(LPC_CLKF_IRC/2000);

    LPC_SYSCTL->SYSOSCCTRL = (LPC_CLKF_OSC < 17000000 ? 0 : 1) << 1;
#endif

#if LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_RTC32K
    /* разрешение RTC клока, если он используется (так как используется только
     * как вход SYSPLL, делать отдельную настройку не имеет смысла */
    LPC_SYSCTL->RTCOSCCTRL = 1;
#endif



#if LPC_USE_SYSPLL
    /* включаем питание PLL */
    LPC_SYSCTL->PDRUNCFG &= ~SYSCTL_POWERDOWN_SYSPLL_PD;

#endif

#ifdef LPC_SYSPLL_SRC
    /* для обновления источника частоты сперва записывается код, затем 0
     * в SYSPLLCLKUEN, затем 1 (UM 4.4.13).
     * Для возможности использовать частоту со входа PLL для системного
     * клока без разрешения самой PLL для этой установки достаточно
     * определения LPC_SYSPLL_SRC (LPC_USE_SYSPLL не обязателен) */
    LPC_SYSCTL->SYSPLLCLKSEL = LPC_SYSPLL_SRC;
    LPC_SYSCTL->SYSPLLCLKUEN = 0;
    LPC_SYSCTL->SYSPLLCLKUEN = 1;
#endif

#if LPC_USE_SYSPLL
    LPC_SYSCTL->SYSPLLCTRL = PLL_CTL_REG_VAL(LPC_SYSPLL_M, LPC_SYSPLL_P);
    while (!(LPC_SYSCTL->SYSPLLSTAT & 1)) {}
#endif

    LPC_SYSCTL->SYSAHBCLKDIV = LPC_SYSCLK_DIV;
    LPC_SYSCTL->MAINCLKSEL = LPC_MAINCLK_SRC;
    LPC_SYSCTL->MAINCLKUEN = 0;
    LPC_SYSCTL->MAINCLKUEN = 1;

#if LPC_USE_USBCLK
    /* USB PLL настраиваем всегда, так как даже если его не используем, то
     * для смены источника частоты USB оба клока должны присутствовать.
     * Если он нам не нежен, то отключим после смены источника клоков */
    LPC_SYSCTL->PDRUNCFG &= ~SYSCTL_POWERDOWN_USBPLL_PD;
    LPC_SYSCTL->USBPLLCLKSEL = LPC_USBPLL_SRC;
    LPC_SYSCTL->USBPLLCLKUEN = 0;
    LPC_SYSCTL->USBPLLCLKUEN = 1;
    LPC_SYSCTL->USBPLLCTRL = PLL_CTL_REG_VAL(LPC_USBPLL_M, LPC_USBPLL_P);
    while (!(LPC_SYSCTL->USBPLLSTAT & 1)) {}

    LPC_SYSCTL->USBCLKSEL = LPC_USBCLK_SRC;
    LPC_SYSCTL->USBCLKUEN = 0;
    LPC_SYSCTL->USBCLKUEN = 1;
    LPC_SYSCTL->USBCLKDIV = LPC_USBCLK_DIV;


#if LPC_USBCLK_SRC != LPC_USBCLK_SRC_USBPLL
    /* если USB PLL не нужен, то можно его тут отключить */
    LPC_SYSCTL->PDRUNCFG |= SYSCTL_POWERDOWN_USBPLL_PD;
#endif
#endif


#if LPC_USE_CLKOUT
    LPC_SYSCTL->CLKOUTSEL = LPC_CLKOUT_SRC;
    LPC_SYSCTL->CLKOUTUEN = 0;
    LPC_SYSCTL->CLKOUTUEN = 1;
    LPC_SYSCTL->CLKOUTDIV = LPC_CLKOUT_DIV;
#if LPC_CLKOUT_DIV != 0
    /* настраиваем пин на вывод частоты */
    LPC_PIN_CONFIG(LPC_PIN_PIO0_1_CLKOUT, 0);
#endif
#endif

    LPC_SYSCTL->USART0CLKDIV = LPC_USART0_CLK_DIV;
    LPC_SYSCTL->SSP0CLKDIV = LPC_SSP0_CLK_DIV;
    LPC_SYSCTL->SSP1CLKDIV = LPC_SSP1_CLK_DIV;

    LPC_SYSCTL->FRGCLKDIV = LPC_FRG_CLK_DIV;
#if LPC_FRG_CLK_DIV
    LPC_SYSCTL->UARTFRGDIV = LPC_USART_FRG_DIV-1;
    LPC_SYSCTL->UARTFRGMULT = LPC_USART_FRG_MUL;
#endif
#if LPC_IRC_PD
    LPC_SYSCTL->PDRUNCFG |= SYSCTL_POWERDOWN_IRC_PD | SYSCTL_POWERDOWN_IRCOUT_PD;
#endif
}
