#ifndef CHIP_INIT_H
#define CHIP_INIT_H

#include "lpc_config.h"

#define LPC_CLKF_RTC        32768UL         /* Частота осциллятора RTC, Гц */
#define LPC_CLKF_IRC        12000000UL      /* Частота внутреннего RC-осциллятора, Гц */

/* источники частоты для системного PLL */
#define LPC_SYSPLL_SRC_IRC     0   /* Internal oscillator in */
#define LPC_SYSPLL_SRC_SYSOSC  1   /* Crystal (main) oscillator in */
#define LPC_SYSPLL_SRC_RTC32K  3   /* RTC 32KHz clock */

/* источники частоты для PLL для USB */
#define LPC_USBPLL_SRC_IRC     0   /* Internal oscillator in */
#define LPC_USBPLL_SRC_SYSOSC  1   /* Crystal (main) oscillator in */


/* источники частоты для главного клока */
#define LPC_MAINCLK_SRC_IRC     0           /* Internal oscillator */
#define LPC_MAINCLK_SRC_PLLIN   1           /* System PLL input */
#define LPC_MAINCLK_SRC_WDTOSC  2           /* Watchdog oscillator rate */
#define LPC_MAINCLK_SRC_PLLOUT  3           /* System PLL output */

/* источники частоты для USB */
#define LPC_USBCLK_SRC_USBPLL    0        /* выход USBPLL */
#define LPC_USBCLK_SRC_MAINCLK   1        /* main clock */

/* источники частоты на выходе CLKOUT */
#define LPC_CLKOUT_SRC_IRC        0  /* IRC oscillator */
#define LPC_CLKOUT_SRC_SYSOSC     1  /* Crystal oscillator */
#define LPC_CLKOUT_SRC_WDTOSC     2  /* Watchdog oscillator */
#define LPC_CLKOUT_SRC_MAINCLK    3  /* main clock */


#define LPC_WDTCLK_SRC_IRC        0
#define LPC_WDTCLK_SRC_WDTOSC     1



#define LPC_WDT_FREQSEL_0_60  0x1 /* 0.6 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_1_05  0x2 /* 1.05 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_1_40  0x3 /* 1.4 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_1_75  0x4 /* 1.75 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_2_10  0x5 /* 2.1 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_2_40  0x6 /* 2.4 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_2_70  0x7 /* 2.7 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_3_00  0x8 /* 3.0 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_3_25  0x9 /* 3.25 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_3_50  0xA /* 3.5 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_3_75  0xB /* 3.75 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_4_00  0xC /* 4.0 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_4_20  0xD /* 4.2 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_4_40  0xE /* 4.4 MHz watchdog/LFO rate */
#define LPC_WDT_FREQSEL_4_60  0xF /* 4.6 MHz watchdog/LFO rate */



#if LPC_SYSPLL_SRC
    #if LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_IRC
        #define LPC_CLKF_SYSPLL_IN LPC_CLKF_IRC
    #elif LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_RTC32K
        #define LPC_CLKF_SYSPLL_IN LPC_CLKF_RTC
    #elif LPC_SYSPLL_SRC == LPC_SYSPLL_SRC_SYSOSC
        #define LPC_CLKF_SYSPLL_IN LPC_CLKF_OSC
    #else
        #error "Invalid value of LPC_SYSPLL_SRC"
    #endif
#endif

#if LPC_USE_SYSPLL
    #define LPC_CLKF_SYSPLL_OUT   (LPC_CLKF_SYSPLL_IN * LPC_SYSPLL_M)
#endif


#if LPC_USE_WDTOSC
    #if LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_0_60
        #define LPC_CLKF_WDT_ANALOG   600000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_1_05
        #define LPC_CLKF_WDT_ANALOG  1050000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_1_40
        #define LPC_CLKF_WDT_ANALOG  1400000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_1_75
        #define LPC_CLKF_WDT_ANALOG  1750000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_2_10
        #define LPC_CLKF_WDT_ANALOG  2100000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_2_40
        #define LPC_CLKF_WDT_ANALOG  2400000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_2_70
        #define LPC_CLKF_WDT_ANALOG  2700000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_3_00
        #define LPC_CLKF_WDT_ANALOG  3000000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_3_25
        #define LPC_CLKF_WDT_ANALOG  3250000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_3_50
        #define LPC_CLKF_WDT_ANALOG  3500000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_3_75
        #define LPC_CLKF_WDT_ANALOG  3750000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_4_00
        #define LPC_CLKF_WDT_ANALOG  4000000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_4_20
        #define LPC_CLKF_WDT_ANALOG  4200000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_4_40
        #define LPC_CLKF_WDT_ANALOG  4400000
    #elif LPC_WDT_FREQSEL == LPC_WDT_FREQSEL_4_60
        #define LPC_CLKF_WDT_ANALOG  4600000
    #else
        #error "Invalid value of LPC_WDT_FREQSEL"
    #endif

    #define LPC_CLKF_WDTOSC  (LPC_CLKF_WDT_ANALOG/(2*(1+LPC_WDT_DIVSEL)))
#endif


#if LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_IRC
    #define LPC_CLKF_MAIN  LPC_CLKF_IRC
#elif LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_PLLIN
    #define LPC_CLKF_MAIN  LPC_CLKF_SYSPLL_IN
#elif LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_WDTOSC
    #define LPC_CLKF_MAIN  LPC_CLKF_WDTOSC
#elif LPC_MAINCLK_SRC == LPC_MAINCLK_SRC_PLLOUT
    #define LPC_CLKF_MAIN LPC_CLKF_SYSPLL_OUT
#endif


#if LPC_USE_USBCLK
    #if LPC_USBPLL_SRC == LPC_USBPLL_SRC_IRC
        #define LPC_CLKF_USBPLL_IN LPC_CLKF_IRC
    #elif LPC_USBPLL_SRC == LPC_USBPLL_SRC_SYSOSC
        #define LPC_CLKF_USBPLL_IN LPC_CLKF_OSC
    #else
        #error "Invalid value of LPC_USBPLL_SRC"
    #endif

    #define LPC_CLKF_USBPLL_OUT (LPC_CLKF_USBPLL_IN * LPC_USBPLL_M)

    #if LPC_USBCLK_SRC == LPC_USBCLK_SRC_USBPLL
        #define LPC_CLKF_USB    LPC_CLKF_USBPLL_OUT/LPC_USBCLK_DIV
    #elif LPC_USBCLK_SRC == LPC_USBCLK_SRC_MAINCLK
        #define LPC_CLKF_USB    LPC_CLKF_MAIN/LPC_USBCLK_DIV
    #else
        #error "Invalid value of LPC_USBCLK_SRC"
    #endif
#endif

#if LPC_USE_CLKOUT
    #if LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_IRC
        #define LPC_CLKF_CLKOUT_SRC  LPC_CLKF_IRC
    #elif LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_SYSOSC
        #define LPC_CLKF_CLKOUT_SRC  LPC_CLKF_OSC
    #elif LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_WDTOSC
        #define LPC_CLKF_CLKOUT_SRC  LPC_CLKF_WDTOSC
    #elif LPC_CLKOUT_SRC == LPC_CLKOUT_SRC_MAINCLK
        #define LPC_CLKF_CLKOUT_SRC  LPC_CLKF_MAIN
    #else
        #error "Invalid value of LPC_CLKOUT_SRC"
    #endif

    #define LPC_CLKF_CLKOUT  LPC_CLKF_CLKOUT_SRC/LPC_CLKOUT_DIV
#endif

#define LPC_SYSCLK  LPC_CLKF_MAIN/LPC_SYSCLK_DIV


/* клоки периферийных устройств */
#define LPC_CLKF_SSP0     LPC_CLKF_MAIN/LPC_SSP0_CLK_DIV
#define LPC_CLKF_SSP1     LPC_CLKF_MAIN/LPC_SSP1_CLK_DIV
#define LPC_CLKF_USART0   LPC_CLKF_MAIN/LPC_USART0_CLK_DIV
#define LPC_CLKF_USART    LPC_CLKF_MAIN/(LPC_FRG_CLK_DIV * (1+ LPC_USART_FRG_MUL/LPC_USART_FRG_DIV))


#if LPC_INIT_WDT
    #if LPC_WDTCLK_SRC == LPC_WDTCLK_SRC_IRC
        #define LPC_CLKF_WDT LPC_CLKF_IRC/4
    #elif LPC_WDTCLK_SRC == LPC_WDTCLK_SRC_WDTOSC
        #define LPC_CLKF_WDT LPC_CLKF_WDTOSC/4
    #else
        #error "Invalid value of LPC_WDTCLK_SRC"
    #endif

    #define LPC_WDT_TICKS_MS(ms)    ((LPC_CLKF_WDT * (ms) + 999) / 1000)
#endif



#if (LPC_SYSCLK > 50000000)
    #error "Invalid system frequency"
#endif


#define LPC_CLK_EN(periph)    (LPC_SYSCTL->SYSAHBCLKCTRL |= (1UL << (periph)))

#define LPC_CLK_DIS(periph)   (LPC_SYSCTL->SYSAHBCLKCTRL &= ~(1UL << (periph)))

#define LPC_CLK_IS_EN(periph) !!(LPC_SYSCTL->SYSAHBCLKCTRL  & (1UL << (periph)))

#define lpc_wdt_reset() do { \
        LPC_WWDT->FEED = 0xAA; \
        LPC_WWDT->FEED = 0x55; \
    } while(0)

#endif // CHIP_INIT_H
