/*================================================================================================*
 * Код начальной инициализации LPC43xx
 *================================================================================================*/
#include <stdint.h>
#include <stddef.h>



/*================================================================================================*/
/* Настройки стека */
#ifndef STACK_SIZE
#error "STACK_SIZE should be defined in makefile"
#endif

//#define STACK_FILLER        0xDEADBEEF

void chip_init(void);

/* Code read protection (в секции .crp_sect) */
#define CRP_DISABLED        0xFFFFFFFF
#define CRP_NO_ISP          0x4E697370
#define CRP1                0x12345678
#define CRP2                0x87654321
#define CRP3                0x43218765

#ifndef CRP_MODE
    #define CRP_MODE            CRP_DISABLED
#endif

/*================================================================================================*/

/*================================================================================================*/
/* Объявление символов, на которые ссылается таблица векторов */
/*================================================================================================*/

#define WEAK_DECL(handler_name) \
    void __attribute__ ((__interrupt__)) handler_name(void) __attribute__ ((__weak__, __alias__("dummy_handler")))

static __attribute__ ((__interrupt__)) void dummy_handler(void);

/* Standard Cortex-M interrupts */
void __attribute__ ((__interrupt__)) Reset_ExHandler(void);
WEAK_DECL(NMI_ExHandler);
WEAK_DECL(HardFault_ExHandler);
WEAK_DECL(SVCall_ExHandler);
WEAK_DECL(PendSV_ExHandler);
WEAK_DECL(SysTick_IRQHandler);
WEAK_DECL(DebugMonitor_IRQHandler);

WEAK_DECL(PIN_INT0_IRQHandler);
WEAK_DECL(PIN_INT1_IRQHandler);
WEAK_DECL(PIN_INT2_IRQHandler);
WEAK_DECL(PIN_INT3_IRQHandler);
WEAK_DECL(PIN_INT4_IRQHandler);
WEAK_DECL(PIN_INT5_IRQHandler);
WEAK_DECL(PIN_INT6_IRQHandler);
WEAK_DECL(PIN_INT7_IRQHandler);
WEAK_DECL(GINT0_IRQHandler);
WEAK_DECL(GINT1_IRQHandler);
WEAK_DECL(I2C1_IRQHandler);
WEAK_DECL(USART1_4_IRQHandler);
WEAK_DECL(USART2_3_IRQHandler);
WEAK_DECL(SCT0_1_IRQHandler);
WEAK_DECL(SSP1_IRQHandler);
WEAK_DECL(I2C0_IRQHandler);
WEAK_DECL(CT16B0_IRQHandler);
WEAK_DECL(CT16B1_IRQHandler);
WEAK_DECL(CT32B0_IRQHandler);
WEAK_DECL(CT32B1_IRQHandler);
WEAK_DECL(SSP0_IRQHandler);
WEAK_DECL(USART0_IRQHandler);
WEAK_DECL(USB_IRQHandler);
WEAK_DECL(USB_FIQHandler);
WEAK_DECL(ADC_A_IRQHandler);
WEAK_DECL(RTC_IRQHandler);
WEAK_DECL(BOD_WDT_IRQHandler);
WEAK_DECL(FLASH_IRQHandler);
WEAK_DECL(DMA_IRQHandler);
WEAK_DECL(ADC_B_IRQHandler);
WEAK_DECL(USBWakeup_IRQHandler);

int main(void);
/*================================================================================================*/

/*================================================================================================*/
/* Символы, определенные линкером (в .ld) */
/*================================================================================================*/
extern uint32_t __data_init_tab_start, __data_init_tab_end;
extern uint32_t __bss_init_tab_start, __bss_init_tab_end;
/*================================================================================================*/

/*================================================================================================*/
/* Стек */
__attribute__ ((section(".stackarea"))) uint32_t g_cpu_stack[STACK_SIZE];

/* Таблица векторов прерываний */
typedef void (*t_irq_handler)(void);

__attribute__ ((section(".isr_vector"))) const t_irq_handler g_int_vectors[] =
    {
    /* IRQ# */
    /* SP */    (t_irq_handler)(g_cpu_stack + STACK_SIZE),
    /* RESET */ Reset_ExHandler,
    /* -14 */   NMI_ExHandler,      HardFault_ExHandler,
    /* -12 */   /* reserved */ NULL, NULL, NULL,
    /*  -9 */   /* reserved */ NULL, NULL, NULL, NULL,
    /*  -5 */   SVCall_ExHandler,   NULL,
    /*  -3 */   /* reserved */ NULL,
    /*  -2 */   PendSV_ExHandler,
    /*  -1 */   SysTick_IRQHandler,

    /*  0 */    PIN_INT0_IRQHandler,    PIN_INT1_IRQHandler,    PIN_INT2_IRQHandler,    PIN_INT3_IRQHandler,
    /*  4 */    PIN_INT4_IRQHandler,    PIN_INT5_IRQHandler,    PIN_INT6_IRQHandler,    PIN_INT7_IRQHandler,
    /*  8 */    GINT0_IRQHandler,       GINT1_IRQHandler,       I2C1_IRQHandler,        USART1_4_IRQHandler,
    /* 12 */    USART2_3_IRQHandler,    SCT0_1_IRQHandler,      SSP1_IRQHandler,        I2C0_IRQHandler,
    /* 16 */    CT16B0_IRQHandler,      CT16B1_IRQHandler,      CT32B0_IRQHandler,      CT32B1_IRQHandler,
    /* 20 */    SSP0_IRQHandler,        USART0_IRQHandler,      USB_IRQHandler,         USB_FIQHandler,
    /* 24 */    ADC_A_IRQHandler,       RTC_IRQHandler,         BOD_WDT_IRQHandler,     FLASH_IRQHandler,
    /* 28 */    DMA_IRQHandler,         ADC_B_IRQHandler,       USBWakeup_IRQHandler,   NULL,
    };

/* CRP */
static __attribute__ ((section(".crp_sect"))) __attribute__ ((used)) const uint32_t f_crp_mode = CRP_MODE;
/*================================================================================================*/

/*================================================================================================*/
/*------------------------------------------------------------------------------------------------*/
static __attribute__ ((__noreturn__)) __attribute__ ((__interrupt__)) void dummy_handler
    (void)
    {
    for (;;);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
void __attribute__ ((__noreturn__)) __attribute__ ((__interrupt__)) Reset_ExHandler(void)
    {
    register const uint32_t* ptab;

    /* Вызвать инициализацию аппаратуры (предже всего PLL) */
    chip_init();

#ifdef STACK_FILLER
    /* Заполнить стек значением STACK_FILLER (для вычисления используемой глубины стека) */
    asm volatile
        (
        "  mov r0, sp \n"
        "0:"
        "  str %[data], [r0, #-4]! \n"
        "  cmp r0, %[stack_start] \n"
        "  bhi 0b"
        :
        : [stack_start] "r" (g_cpu_stack), [data] "r" (STACK_FILLER)
        : "r0", "cc", "memory"
        );
#endif

    /* Копирование инициализированных данных */
    /* Формат записи списка секций: { src, dest, len } */
    for (ptab = &__data_init_tab_start; ptab < &__data_init_tab_end; )
        {
        register const uint32_t *ps = (const uint32_t*)*ptab++;
        register uint32_t* pd = (uint32_t*)*ptab++;
        register uint32_t* pend = (uint32_t*)((uint32_t)pd + *ptab++);
        while (pd < pend)
            {
            *pd++ = *ps++;
            }
        }

    /* Обнуление bss */
    /* Формат записи списка секций: { dest, len } */
    for (ptab = &__bss_init_tab_start; ptab < &__bss_init_tab_end; )
        {
        register uint32_t* pd = (uint32_t*)*ptab++;
        register uint32_t* pend = (uint32_t*)((uint32_t)pd + *ptab++);
        while (pd < pend)
            {
            *pd++ = 0;
            }
        }

    /* Вызов основной программы */
    main();

    for (;;);
    }
/*------------------------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------------------------*/
unsigned int sys_stack_watermark    /* Оценка максимальной глубины использованного стека */
    (void)
    {
    unsigned int index = 0;
#ifdef STACK_FILLER
    /* Считаем, что если слово в стеке содержит начальное значение STACK_FILLER, то оно, вероятно,
       никогда не использовалось */
    while ((index < STACK_SIZE) && (STACK_FILLER == g_cpu_stack[index]))
        {
        index++;
        }
#endif
    return index;
    }
/*------------------------------------------------------------------------------------------------*/
/*================================================================================================*/
