#ifndef LPC_PIN_H
#define LPC_PIN_H

#define LPC_PIN_ID(port, pin, func)   (((port&0xFF)<<16) | ((pin&0xFF) << 8) | (func&0xFF))

/* Извлечение полей ID (internal use) */
#define LPC_PIN_ID_PORT(id) ((id>>16) & 0xFF)
#define LPC_PIN_ID_PIN(id)  ((id>>8) & 0xFF)
#define LPC_PIN_ID_FINC(id) (id & 0xFF)


/* Адресация регистров LPC по pin ID */
#define LPC_PIN_PIOREG_IOCON(id)     (LPC_PIN_ID_PORT(id) == 0 ? &LPC_IOCON->PIO0[LPC_PIN_ID_PIN(id)] : \
                                  LPC_PIN_ID_PORT(id) == 1 ? &LPC_IOCON->PIO1[LPC_PIN_ID_PIN(id)] : \
                                  (LPC_PIN_ID_PORT(id) == 2) && (LPC_PIN_ID_PIN(id)<2) ?  \
                                        &LPC_IOCON->PIO2A[LPC_PIN_ID_PIN(id) < 2 ? LPC_PIN_ID_PIN(id)  : 0] : \
                                        &LPC_IOCON->PIO2B[LPC_PIN_ID_PIN(id)-2])


#define LPC_PIN_CONFIG_(id, iocon_mode) do { \    
    *LPC_PIN_PIOREG_IOCON(id) = (LPC_PIN_ID_FINC(id) & 0x7) | ((iocon_mode) & 0xFFFFFFF8); \
    } while(0)

#define LPC_PIN_CONFIG(id, iocon_mode) do { \    
    int iocon_is_en = LPC_CLK_IS_EN(SYSCTL_CLOCK_IOCON); \
    LPC_CLK_EN(SYSCTL_CLOCK_IOCON); \
    LPC_PIN_CONFIG_((id), (iocon_mode)); \
    if (!iocon_is_en) \
        LPC_CLK_DIS(SYSCTL_CLOCK_IOCON); \
    } while(0)

#define LPC_PIN_GPIOREG(reg, id)    LPC_GPIO->reg[LPC_PIN_ID_PORT(id)]
#define LPC_PIN_GPIOB(id)           LPC_GPIO->B[LPC_PIN_ID_PORT(id)][LPC_PIN_ID_PIN(id)]
#define LPC_PIN_BITMSK(id)          (1UL << LPC_PIN_ID_PIN(id))



/* Управление ножками GPIO */
    /* управление направлением */
#define LPC_PIN_DIR_GET(id)         (!!(LPC_PIN_GPIOREG(DIR, (id)) & LPC_PIN_BITMSK(id)))
#define LPC_PIN_DIR_IN(id)          (LPC_PIN_GPIOREG(DIR, (id)) &= ~LPC_PIN_BITMSK(id))
#define LPC_PIN_DIR_OUT(id)         (LPC_PIN_GPIOREG(DIR, (id)) |= LPC_PIN_BITMSK(id))
    /* установка уровня на ножке (0 или 1) */
#define LPC_PIN_OUT(id, val)        (LPC_PIN_GPIOB(id) = (val))
    /* чтение уровня на ножке (0 или 1) */
#define LPC_PIN_IN(id)              LPC_PIN_GPIOB(id)
    /* атомарное инвертирование уровня на ножке */
#define LPC_PIN_TOGGLE(id)          (LPC_PIN_GPIOREG(NOT, (id)) = LPC_PIN_BITMSK(id))
    /* состояние регистра выхода (независимо от физического уровня) */
#define LPC_PIN_GET_OUT_STATE(id)   (!!(LPC_PIN_GPIOREG(SET, (id)) & LPC_PIN_BITMSK(id)))


/* Список функций ножек p89 */

/* Port 0 */
#define LPC_PIN_PIO0_0_RESET        LPC_PIN_ID(0,0,0)
#define LPC_PIN_PIO0_0_PIO          LPC_PIN_ID(0,0,1)

#define LPC_PIN_PIO0_1_PIO          LPC_PIN_ID(0,1,0)
#define LPC_PIN_PIO0_1_CLKOUT       LPC_PIN_ID(0,1,1)
#define LPC_PIN_PIO0_1_CT32B0_MAT2  LPC_PIN_ID(0,1,2)
#define LPC_PIN_PIO0_1_USB_FTOGGLE  LPC_PIN_ID(0,1,3)

#define LPC_PIN_PIO0_2_PIO          LPC_PIN_ID(0,2,0)
#define LPC_PIN_PIO0_2_SSP0_SSEL    LPC_PIN_ID(0,2,1)
#define LPC_PIN_PIO0_2_CT16B0_CAP0  LPC_PIN_ID(0,2,2)
#define LPC_PIN_PIO0_2_R_0          LPC_PIN_ID(0,2,3)

#define LPC_PIN_PIO0_3_PIO          LPC_PIN_ID(0,3,0)
#define LPC_PIN_PIO0_3_USB_VBUS     LPC_PIN_ID(0,3,1)
#define LPC_PIN_PIO0_3_R_1          LPC_PIN_ID(0,3,2)

#define LPC_PIN_PIO0_4_PIO          LPC_PIN_ID(0,4,0)
#define LPC_PIN_PIO0_4_I2C0_SCL     LPC_PIN_ID(0,4,1)
#define LPC_PIN_PIO0_4_R_2          LPC_PIN_ID(0,4,2)

#define LPC_PIN_PIO0_5_PIO          LPC_PIN_ID(0,5,0)
#define LPC_PIN_PIO0_5_I2C0_SDA     LPC_PIN_ID(0,5,1)
#define LPC_PIN_PIO0_5_R_3          LPC_PIN_ID(0,5,2)

#define LPC_PIN_PIO0_6_PIO          LPC_PIN_ID(0,6,0)
#define LPC_PIN_PIO0_6_R            LPC_PIN_ID(0,6,1)
#define LPC_PIN_PIO0_6_SSP0_SCK     LPC_PIN_ID(0,6,2)
#define LPC_PIN_PIO0_6_R_4          LPC_PIN_ID(0,6,3)

#define LPC_PIN_PIO0_7_PIO          LPC_PIN_ID(0,7,0)
#define LPC_PIN_PIO0_7_U0_CTS       LPC_PIN_ID(0,7,1)
#define LPC_PIN_PIO0_7_R_5          LPC_PIN_ID(0,7,2)
#define LPC_PIN_PIO0_7_I2C1_SCL     LPC_PIN_ID(0,7,3)

#define LPC_PIN_PIO0_8_PIO          LPC_PIN_ID(0,8,0)
#define LPC_PIN_PIO0_8_SSP0_MISO    LPC_PIN_ID(0,8,1)
#define LPC_PIN_PIO0_8_CT16B0_MAT0  LPC_PIN_ID(0,8,2)
#define LPC_PIN_PIO0_8_R_6          LPC_PIN_ID(0,8,3)

#define LPC_PIN_PIO0_9_PIO          LPC_PIN_ID(0,9,0)
#define LPC_PIN_PIO0_9_SSP0_MOSI    LPC_PIN_ID(0,9,1)
#define LPC_PIN_PIO0_9_CT16B0_MAT1  LPC_PIN_ID(0,9,2)
#define LPC_PIN_PIO0_9_R_7          LPC_PIN_ID(0,9,3)

#define LPC_PIN_PIO0_10_SWCLK       LPC_PIN_ID(0,10,0)
#define LPC_PIN_PIO0_10_PIO         LPC_PIN_ID(0,10,1)
#define LPC_PIN_PIO0_10_SSP0_SCK    LPC_PIN_ID(0,10,2)
#define LPC_PIN_PIO0_10_CT16B0_MAT2 LPC_PIN_ID(0,10,3)

#define LPC_PIN_PIO0_11_TDI         LPC_PIN_ID(0,11,0)
#define LPC_PIN_PIO0_11_PIO         LPC_PIN_ID(0,11,1)
#define LPC_PIN_PIO0_11_ADC_9       LPC_PIN_ID(0,11,2)
#define LPC_PIN_PIO0_11_CT32B0_MAT3 LPC_PIN_ID(0,11,3)
#define LPC_PIN_PIO0_11_U1_RTS      LPC_PIN_ID(0,11,4)

#define LPC_PIN_PIO0_12_TMS         LPC_PIN_ID(0,12,0)
#define LPC_PIN_PIO0_12_PIO         LPC_PIN_ID(0,12,1)
#define LPC_PIN_PIO0_12_ADC_8       LPC_PIN_ID(0,12,2)
#define LPC_PIN_PIO0_12_CT32B1_CAP0 LPC_PIN_ID(0,12,3)
#define LPC_PIN_PIO0_12_U1_CTS      LPC_PIN_ID(0,12,4)

#define LPC_PIN_PIO0_13_TDO         LPC_PIN_ID(0,13,0)
#define LPC_PIN_PIO0_13_PIO         LPC_PIN_ID(0,13,1)
#define LPC_PIN_PIO0_13_ADC_7       LPC_PIN_ID(0,13,2)
#define LPC_PIN_PIO0_13_CT32B1_MAT0 LPC_PIN_ID(0,13,3)
#define LPC_PIN_PIO0_13_U1_RXD      LPC_PIN_ID(0,13,4)

#define LPC_PIN_PIO0_14_TRST        LPC_PIN_ID(0,14,0)
#define LPC_PIN_PIO0_14_PIO         LPC_PIN_ID(0,14,1)
#define LPC_PIN_PIO0_14_ADC_6       LPC_PIN_ID(0,14,2)
#define LPC_PIN_PIO0_14_CT32B1_MAT1 LPC_PIN_ID(0,14,3)
#define LPC_PIN_PIO0_14_U1_TXD      LPC_PIN_ID(0,14,4)

#define LPC_PIN_PIO0_15_SWDIO       LPC_PIN_ID(0,15,0)
#define LPC_PIN_PIO0_15_PIO         LPC_PIN_ID(0,15,1)
#define LPC_PIN_PIO0_15_ADC_3       LPC_PIN_ID(0,15,2)
#define LPC_PIN_PIO0_15_CT32B1_MAT2 LPC_PIN_ID(0,15,3)

#define LPC_PIN_PIO0_16_PIO         LPC_PIN_ID(0,16,0)
#define LPC_PIN_PIO0_16_ADC_2       LPC_PIN_ID(0,16,1)
#define LPC_PIN_PIO0_16_CT32B1_MAT3 LPC_PIN_ID(0,16,2)
#define LPC_PIN_PIO0_16_R_8         LPC_PIN_ID(0,16,3)

#define LPC_PIN_PIO0_17_PIO         LPC_PIN_ID(0,17,0)
#define LPC_PIN_PIO0_17_U0_RTS      LPC_PIN_ID(0,17,1)
#define LPC_PIN_PIO0_17_CT32B0_CAP0 LPC_PIN_ID(0,17,2)
#define LPC_PIN_PIO0_17_U0_SCLK     LPC_PIN_ID(0,17,3)

#define LPC_PIN_PIO0_18_PIO         LPC_PIN_ID(0,18,0)
#define LPC_PIN_PIO0_18_U0_RXD      LPC_PIN_ID(0,18,1)
#define LPC_PIN_PIO0_18_CT32B0_MAT0 LPC_PIN_ID(0,18,2)

#define LPC_PIN_PIO0_19_PIO         LPC_PIN_ID(0,19,0)
#define LPC_PIN_PIO0_19_U0_TXD      LPC_PIN_ID(0,19,1)
#define LPC_PIN_PIO0_19_CT32B0_MAT1 LPC_PIN_ID(0,19,2)

#define LPC_PIN_PIO0_20_PIO         LPC_PIN_ID(0,20,0)
#define LPC_PIN_PIO0_20_CT16B1_CAP0 LPC_PIN_ID(0,20,1)
#define LPC_PIN_PIO0_20_U2_RXD      LPC_PIN_ID(0,20,2)

#define LPC_PIN_PIO0_21_PIO         LPC_PIN_ID(0,21,0)
#define LPC_PIN_PIO0_21_CT16B1_MAT0 LPC_PIN_ID(0,21,1)
#define LPC_PIN_PIO0_21_SSP1_MOSI   LPC_PIN_ID(0,21,2)

#define LPC_PIN_PIO0_22_PIO         LPC_PIN_ID(0,22,0)
#define LPC_PIN_PIO0_22_ADC_11      LPC_PIN_ID(0,22,1)
#define LPC_PIN_PIO0_22_CT16B1_CAP1 LPC_PIN_ID(0,22,2)
#define LPC_PIN_PIO0_22_SSP1_MISO   LPC_PIN_ID(0,22,3)

#define LPC_PIN_PIO0_23_PIO         LPC_PIN_ID(0,23,0)
#define LPC_PIN_PIO0_23_ADC_1       LPC_PIN_ID(0,23,1)
#define LPC_PIN_PIO0_23_R_9         LPC_PIN_ID(0,23,2)
#define LPC_PIN_PIO0_23_U0_RI       LPC_PIN_ID(0,23,3)
#define LPC_PIN_PIO0_23_SSP1_SSEL   LPC_PIN_ID(0,23,4)

/* Port 1 */
#define LPC_PIN_PIO1_0_PIO          LPC_PIN_ID(1,0,0)
#define LPC_PIN_PIO1_0_CT32B1_MAT0  LPC_PIN_ID(1,0,1)
#define LPC_PIN_PIO1_0_R_10         LPC_PIN_ID(1,0,2)
#define LPC_PIN_PIO1_0_U2_TXD       LPC_PIN_ID(1,0,3)

#define LPC_PIN_PIO1_1_PIO          LPC_PIN_ID(1,1,0)
#define LPC_PIN_PIO1_1_CT32B1_MAT1  LPC_PIN_ID(1,1,1)
#define LPC_PIN_PIO1_1_R_11         LPC_PIN_ID(1,1,2)
#define LPC_PIN_PIO1_1_U0_DTR       LPC_PIN_ID(1,1,3)

#define LPC_PIN_PIO1_2_PIO          LPC_PIN_ID(1,2,0)
#define LPC_PIN_PIO1_2_CT32B1_MAT2  LPC_PIN_ID(1,2,1)
#define LPC_PIN_PIO1_2_R_12         LPC_PIN_ID(1,2,2)
#define LPC_PIN_PIO1_2_U1_RXD       LPC_PIN_ID(1,2,3)

#define LPC_PIN_PIO1_3_PIO          LPC_PIN_ID(1,3,0)
#define LPC_PIN_PIO1_3_CT32B1_MAT3  LPC_PIN_ID(1,3,1)
#define LPC_PIN_PIO1_3_R_13         LPC_PIN_ID(1,3,2)
#define LPC_PIN_PIO1_3_I2C1_SDA     LPC_PIN_ID(1,3,3)
#define LPC_PIN_PIO1_3_ADC_5        LPC_PIN_ID(1,3,4)

#define LPC_PIN_PIO1_4_PIO          LPC_PIN_ID(1,4,0)
#define LPC_PIN_PIO1_4_CT32B1_CAP0  LPC_PIN_ID(1,4,1)
#define LPC_PIN_PIO1_4_R_14         LPC_PIN_ID(1,4,2)
#define LPC_PIN_PIO1_4_U0_DSR       LPC_PIN_ID(1,4,3)

#define LPC_PIN_PIO1_5_PIO          LPC_PIN_ID(1,5,0)
#define LPC_PIN_PIO1_5_CT32B1_CAP1  LPC_PIN_ID(1,5,1)
#define LPC_PIN_PIO1_5_R_15         LPC_PIN_ID(1,5,2)
#define LPC_PIN_PIO1_5_U0_DCD       LPC_PIN_ID(1,5,3)

#define LPC_PIN_PIO1_6_PIO          LPC_PIN_ID(1,6,0)
#define LPC_PIN_PIO1_6_R_16         LPC_PIN_ID(1,6,1)
#define LPC_PIN_PIO1_6_U2_RXD       LPC_PIN_ID(1,6,2)
#define LPC_PIN_PIO1_6_CT32B0_CAP1  LPC_PIN_ID(1,6,3)

#define LPC_PIN_PIO1_7_PIO          LPC_PIN_ID(1,7,0)
#define LPC_PIN_PIO1_7_R_17         LPC_PIN_ID(1,7,1)
#define LPC_PIN_PIO1_7_U2_CTS       LPC_PIN_ID(1,7,2)
#define LPC_PIN_PIO1_7_CT32B1_CAP0  LPC_PIN_ID(1,7,3)

#define LPC_PIN_PIO1_8_PIO          LPC_PIN_ID(1,8,0)
#define LPC_PIN_PIO1_8_R_18         LPC_PIN_ID(1,8,1)
#define LPC_PIN_PIO1_8_U1_TXD       LPC_PIN_ID(1,8,2)
#define LPC_PIN_PIO1_8_CT16B0_CAP0  LPC_PIN_ID(1,8,3)

#define LPC_PIN_PIO1_9_PIO          LPC_PIN_ID(1,9,0)
#define LPC_PIN_PIO1_9_U0_CTS       LPC_PIN_ID(1,9,1)
#define LPC_PIN_PIO1_9_CT16b1_MAT1  LPC_PIN_ID(1,9,2)
#define LPC_PIN_PIO1_9_ADC_0        LPC_PIN_ID(1,9,3)

#define LPC_PIN_PIO1_10_PIO         LPC_PIN_ID(1,10,0)
#define LPC_PIN_PIO1_10_U2_RTS      LPC_PIN_ID(1,10,1)
#define LPC_PIN_PIO1_10_U2_SCLK     LPC_PIN_ID(1,10,2)
#define LPC_PIN_PIO1_10_CT16B1_MAT0 LPC_PIN_ID(1,10,3)

#define LPC_PIN_PIO1_11_PIO         LPC_PIN_ID(1,11,0)
#define LPC_PIN_PIO1_11_I2C1_SCL    LPC_PIN_ID(1,11,1)
#define LPC_PIN_PIO1_11_CT16B0_MAT2 LPC_PIN_ID(1,11,2)
#define LPC_PIN_PIO1_11_U0_RI       LPC_PIN_ID(1,11,3)

#define LPC_PIN_PIO1_12_PIO         LPC_PIN_ID(1,12,0)
#define LPC_PIN_PIO1_12_SSP0_MOSI   LPC_PIN_ID(1,12,1)
#define LPC_PIN_PIO1_12_CT16B0_MAT1 LPC_PIN_ID(1,12,2)
#define LPC_PIN_PIO1_12_R_21        LPC_PIN_ID(1,12,3)

#define LPC_PIN_PIO1_13_PIO         LPC_PIN_ID(1,13,0)
#define LPC_PIN_PIO1_13_U1_CTS      LPC_PIN_ID(1,13,1)
#define LPC_PIN_PIO1_13_SCT0_OUT3   LPC_PIN_ID(1,13,2)
#define LPC_PIN_PIO1_13_R_22        LPC_PIN_ID(1,13,3)

#define LPC_PIN_PIO1_14_PIO         LPC_PIN_ID(1,14,0)
#define LPC_PIN_PIO1_14_I2C1_SDA    LPC_PIN_ID(1,14,1)
#define LPC_PIN_PIO1_14_CT32B1_MAT2 LPC_PIN_ID(1,14,2)
#define LPC_PIN_PIO1_14_R_23        LPC_PIN_ID(1,14,3)

#define LPC_PIN_PIO1_15_PIO         LPC_PIN_ID(1,15,0)
#define LPC_PIN_PIO1_15_SSP0_SSEL   LPC_PIN_ID(1,15,1)
#define LPC_PIN_PIO1_15_CT32B1_MAT3 LPC_PIN_ID(1,15,2)
#define LPC_PIN_PIO1_15_R_24        LPC_PIN_ID(1,15,3)

#define LPC_PIN_PIO1_16_PIO         LPC_PIN_ID(1,16,0)
#define LPC_PIN_PIO1_16_SSP0_MISO   LPC_PIN_ID(1,16,1)
#define LPC_PIN_PIO1_16_CT16B0_MAT0 LPC_PIN_ID(1,16,2)
#define LPC_PIN_PIO1_16_R_25        LPC_PIN_ID(1,16,3)

#define LPC_PIN_PIO1_17_PIO         LPC_PIN_ID(1,17,0)
#define LPC_PIN_PIO1_17_CT16B0_CAP2 LPC_PIN_ID(1,17,1)
#define LPC_PIN_PIO1_17_U0_RXD      LPC_PIN_ID(1,17,2)
#define LPC_PIN_PIO1_17_R_26        LPC_PIN_ID(1,17,3)

#define LPC_PIN_PIO1_18_PIO         LPC_PIN_ID(1,18,0)
#define LPC_PIN_PIO1_18_CT16B1_CAP1 LPC_PIN_ID(1,18,1)
#define LPC_PIN_PIO1_18_U0_TXD      LPC_PIN_ID(1,18,2)
#define LPC_PIN_PIO1_18_R_27        LPC_PIN_ID(1,18,3)

#define LPC_PIN_PIO1_19_PIO         LPC_PIN_ID(1,19,0)
#define LPC_PIN_PIO1_19_U2_CTS      LPC_PIN_ID(1,19,1)
#define LPC_PIN_PIO1_19_SCT0_OUT0   LPC_PIN_ID(1,19,2)
#define LPC_PIN_PIO1_19_R_28        LPC_PIN_ID(1,19,3)

#define LPC_PIN_PIO1_20_PIO         LPC_PIN_ID(1,20,0)
#define LPC_PIN_PIO1_20_U0_DSR      LPC_PIN_ID(1,20,1)
#define LPC_PIN_PIO1_20_SSP1_SCK    LPC_PIN_ID(1,20,2)
#define LPC_PIN_PIO1_20_CT16B0_MAT0 LPC_PIN_ID(1,20,3)

#define LPC_PIN_PIO1_21_PIO         LPC_PIN_ID(1,21,0)
#define LPC_PIN_PIO1_21_U0_DCD      LPC_PIN_ID(1,21,1)
#define LPC_PIN_PIO1_21_SSP1_MISO   LPC_PIN_ID(1,21,2)
#define LPC_PIN_PIO1_21_CT16B0_CAP1 LPC_PIN_ID(1,21,3)

#define LPC_PIN_PIO1_22_PIO         LPC_PIN_ID(1,22,0)
#define LPC_PIN_PIO1_22_SSP1_MOSI   LPC_PIN_ID(1,22,1)
#define LPC_PIN_PIO1_22_CT32B1_CAP1 LPC_PIN_ID(1,22,2)
#define LPC_PIN_PIO1_22_ADC_4       LPC_PIN_ID(1,22,3)
#define LPC_PIN_PIO1_22_R_29        LPC_PIN_ID(1,22,4)

#define LPC_PIN_PIO1_23_PIO         LPC_PIN_ID(1,23,0)
#define LPC_PIN_PIO1_23_CT16B1_MAT1 LPC_PIN_ID(1,23,1)
#define LPC_PIN_PIO1_23_SSP1_SSEL   LPC_PIN_ID(1,23,2)
#define LPC_PIN_PIO1_23_U2_TXD      LPC_PIN_ID(1,23,3)

#define LPC_PIN_PIO1_24_PIO         LPC_PIN_ID(1,24,0)
#define LPC_PIN_PIO1_24_CT32B0_MAT0 LPC_PIN_ID(1,24,1)
#define LPC_PIN_PIO1_24_I2C1_SDA    LPC_PIN_ID(1,24,2)

#define LPC_PIN_PIO1_25_PIO         LPC_PIN_ID(1,25,0)
#define LPC_PIN_PIO1_25_U2_RTS      LPC_PIN_ID(1,25,1)
#define LPC_PIN_PIO1_25_U2_SCLK     LPC_PIN_ID(1,25,2)
#define LPC_PIN_PIO1_25_SCT0_IN0    LPC_PIN_ID(1,25,3)
#define LPC_PIN_PIO1_25_R_30        LPC_PIN_ID(1,25,4)

#define LPC_PIN_PIO1_26_PIO         LPC_PIN_ID(1,26,0)
#define LPC_PIN_PIO1_26_CT32B0_MAT2 LPC_PIN_ID(1,26,1)
#define LPC_PIN_PIO1_26_U0_RXD      LPC_PIN_ID(1,26,2)
#define LPC_PIN_PIO1_26_R_19        LPC_PIN_ID(1,26,3)

#define LPC_PIN_PIO1_27_PIO         LPC_PIN_ID(1,27,0)
#define LPC_PIN_PIO1_27_CT32B0_MAT3 LPC_PIN_ID(1,27,1)
#define LPC_PIN_PIO1_27_U0_TXD      LPC_PIN_ID(1,27,2)
#define LPC_PIN_PIO1_27_R_20        LPC_PIN_ID(1,27,3)
#define LPC_PIN_PIO1_27_SSP1_SCK    LPC_PIN_ID(1,27,4)

#define LPC_PIN_PIO1_28_PIO         LPC_PIN_ID(1,28,0)
#define LPC_PIN_PIO1_28_CT32B0_CAP0 LPC_PIN_ID(1,28,1)
#define LPC_PIN_PIO1_28_U0_SCLK     LPC_PIN_ID(1,28,2)
#define LPC_PIN_PIO1_28_U0_RTS      LPC_PIN_ID(1,28,3)

#define LPC_PIN_PIO1_29_PIO         LPC_PIN_ID(1,29,0)
#define LPC_PIN_PIO1_29_SSP0_SCK    LPC_PIN_ID(1,29,1)
#define LPC_PIN_PIO1_29_CT32B0_CAP1 LPC_PIN_ID(1,29,2)
#define LPC_PIN_PIO1_29_U0_DTRn     LPC_PIN_ID(1,29,3)
#define LPC_PIN_PIO1_29_ADC_10      LPC_PIN_ID(1,29,4)

#define LPC_PIN_PIO1_30_PIO         LPC_PIN_ID(1,30,0)
#define LPC_PIN_PIO1_30_I2C1_SCL    LPC_PIN_ID(1,30,1)
#define LPC_PIN_PIO1_30_SCT0_IN3    LPC_PIN_ID(1,30,2)
#define LPC_PIN_PIO1_30_R_31        LPC_PIN_ID(1,30,3)

#define LPC_PIN_PIO1_31_PIO         LPC_PIN_ID(1,31,0)


/* Port 2 */
#define LPC_PIN_PIO2_0_PIO          LPC_PIN_ID(2,0,0)
#define LPC_PIN_PIO2_0_XTALIN       LPC_PIN_ID(2,0,1)

#define LPC_PIN_PIO2_1_PIO          LPC_PIN_ID(2,1,0)
#define LPC_PIN_PIO2_1_XTALOUT      LPC_PIN_ID(2,1,1)

#define LPC_PIN_PIO2_2_PIO          LPC_PIN_ID(2,2,0)
#define LPC_PIN_PIO2_2_U3_RTS       LPC_PIN_ID(2,2,1)
#define LPC_PIN_PIO2_2_U3_SCLK      LPC_PIN_ID(2,2,2)
#define LPC_PIN_PIO2_2_SCT0_OUT1    LPC_PIN_ID(2,2,3)

#define LPC_PIN_PIO2_3_PIO          LPC_PIN_ID(2,3,0)
#define LPC_PIN_PIO2_3_U3_RXD       LPC_PIN_ID(2,3,1)
#define LPC_PIN_PIO2_3_CT32B0_MAT1  LPC_PIN_ID(2,3,2)

#define LPC_PIN_PIO2_4_PIO          LPC_PIN_ID(2,4,0)
#define LPC_PIN_PIO2_4_U3_TXD       LPC_PIN_ID(2,4,1)
#define LPC_PIN_PIO2_4_CT32B0_MAT2  LPC_PIN_ID(2,4,2)

#define LPC_PIN_PIO2_5_PIO          LPC_PIN_ID(2,5,0)
#define LPC_PIN_PIO2_5_U3_CTS       LPC_PIN_ID(2,5,1)
#define LPC_PIN_PIO2_5_SCT0_IN1     LPC_PIN_ID(2,5,2)

#define LPC_PIN_PIO2_6_PIO          LPC_PIN_ID(2,6,0)
#define LPC_PIN_PIO2_6_U1_RTS       LPC_PIN_ID(2,6,1)
#define LPC_PIN_PIO2_6_U1_SCLK      LPC_PIN_ID(2,6,2)
#define LPC_PIN_PIO2_6_SCT0_IN2     LPC_PIN_ID(2,6,3)

#define LPC_PIN_PIO2_7_PIO          LPC_PIN_ID(2,7,0)
#define LPC_PIN_PIO2_7_SSP0_SCK     LPC_PIN_ID(2,7,1)
#define LPC_PIN_PIO2_7_SCT0_OUT2    LPC_PIN_ID(2,7,2)

#define LPC_PIN_PIO2_8_PIO          LPC_PIN_ID(2,8,0)
#define LPC_PIN_PIO2_8_SCT1_IN0     LPC_PIN_ID(2,8,1)

#define LPC_PIN_PIO2_9_PIO          LPC_PIN_ID(2,9,0)
#define LPC_PIN_PIO2_9_SCT1_IN1     LPC_PIN_ID(2,9,1)

#define LPC_PIN_PIO2_10_PIO         LPC_PIN_ID(2,10,0)
#define LPC_PIN_PIO2_10_U4_RTS      LPC_PIN_ID(2,10,1)
#define LPC_PIN_PIO2_10_U4_SCLK     LPC_PIN_ID(2,10,2)

#define LPC_PIN_PIO2_11_PIO         LPC_PIN_ID(2,11,0)
#define LPC_PIN_PIO2_11_U4_RXD      LPC_PIN_ID(2,11,1)

#define LPC_PIN_PIO2_12_PIO         LPC_PIN_ID(2,12,0)
#define LPC_PIN_PIO2_12_U4_TXD      LPC_PIN_ID(2,12,1)

#define LPC_PIN_PIO2_13_PIO         LPC_PIN_ID(2,13,0)
#define LPC_PIN_PIO2_13_U4_CTS      LPC_PIN_ID(2,13,1)

#define LPC_PIN_PIO2_14_PIO         LPC_PIN_ID(2,14,0)
#define LPC_PIN_PIO2_14_SCT1_IN2    LPC_PIN_ID(2,14,1)

#define LPC_PIN_PIO2_15_PIO         LPC_PIN_ID(2,15,0)
#define LPC_PIN_PIO2_15_SCT1_IN3    LPC_PIN_ID(2,15,1)

#define LPC_PIN_PIO2_16_PIO         LPC_PIN_ID(2,16,0)
#define LPC_PIN_PIO2_16_SCT1_OUT0   LPC_PIN_ID(2,16,1)

#define LPC_PIN_PIO2_17_PIO         LPC_PIN_ID(2,17,0)
#define LPC_PIN_PIO2_17_SCT1_OUT1   LPC_PIN_ID(2,17,1)

#define LPC_PIN_PIO2_18_PIO         LPC_PIN_ID(2,18,0)
#define LPC_PIN_PIO2_18_SCT1_OUT2   LPC_PIN_ID(2,18,1)

#define LPC_PIN_PIO2_19_PIO         LPC_PIN_ID(2,19,0)
#define LPC_PIN_PIO2_19_SCT1_OUT3   LPC_PIN_ID(2,19,1)

#define LPC_PIN_PIO2_20_PIO         LPC_PIN_ID(2,20,0)

#define LPC_PIN_PIO2_21_PIO         LPC_PIN_ID(2,21,0)

#define LPC_PIN_PIO2_22_PIO         LPC_PIN_ID(2,22,0)

#define LPC_PIN_PIO2_23_PIO         LPC_PIN_ID(2,23,0)

#endif // LPC_PIN_H
