/*
 * @brief LPC11U6X Clock control functions
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __CLOCK_11U6X_H_
#define __CLOCK_11U6X_H_

/**
 * System and peripheral clocks
 */
typedef enum CHIP_SYSCTL_CLOCK {
    SYSCTL_CLOCK_SYS = 0,                   /*!< 0: System clock */
    SYSCTL_CLOCK_ROM,                       /*!< 1:  ROM clock */
    SYSCTL_CLOCK_RAM0,                      /*!< 2: RAM0 clock */
    SYSCTL_CLOCK_FLASHREG,                  /*!< 3: FLASH register interface clock */
    SYSCTL_CLOCK_FLASHARRAY,                /*!< 4: FLASH array access clock */
    SYSCTL_CLOCK_I2C0,                      /*!< 5: I2C0 clock */
    SYSCTL_CLOCK_GPIO,                      /*!< 6: GPIO clock */
    SYSCTL_CLOCK_CT16B0,                    /*!< 7: 16-bit Counter/timer 0 clock */
    SYSCTL_CLOCK_CT16B1,                    /*!< 8: 16-bit Counter/timer 1 clock */
    SYSCTL_CLOCK_CT32B0,                    /*!< 9: 32-bit Counter/timer 0 clock */
    SYSCTL_CLOCK_CT32B1,                    /*!< 10: 32-bit Counter/timer 1 clock */
    SYSCTL_CLOCK_SSP0,                      /*!< 11: SSP0 clock */
    SYSCTL_CLOCK_UART0,                     /*!< 12: UART0 clock */
    SYSCTL_CLOCK_ADC,                       /*!< 13: ADC clock */
    SYSCTL_CLOCK_USB,                       /*!< 14: USB clock */
    SYSCTL_CLOCK_WDT,                       /*!< 15: Watchdog timer clock */
    SYSCTL_CLOCK_IOCON,                     /*!< 16: IOCON block clock */
    SYSCTL_CLOCK_RESERVED17,                /*!< 17: Reserved */
    SYSCTL_CLOCK_SSP1,                      /*!< 18: SSP1 clock */
    SYSCTL_CLOCK_PINT,                      /*!< 19: GPIO Pin int register interface clock */
    SYSCTL_CLOCK_USART1,                    /*!< 20: USART1 clock */
    SYSCTL_CLOCK_USART2,                    /*!< 21: USART2 clock */
    SYSCTL_CLOCK_USART3_4,                  /*!< 22: USART3_4 clock */
    SYSCTL_CLOCK_P0INT,                     /*!< 23: GPIO GROUP1 interrupt register clock */
    SYSCTL_CLOCK_GROUP0INT = SYSCTL_CLOCK_P0INT,/*!< 23: GPIO GROUP0 interrupt register interface clock */
    SYSCTL_CLOCK_P1INT,                     /*!< 24: GPIO GROUP1 interrupt register clock */
    SYSCTL_CLOCK_GROUP1INT = SYSCTL_CLOCK_P1INT,/*!< 24: GPIO GROUP1 interrupt register interface clock */
    SYSCTL_CLOCK_I2C1,                      /*!< 25: I2C1 clock */
    SYSCTL_CLOCK_RAM1,                      /*!< 26: SRAM block clock */
    SYSCTL_CLOCK_USBRAM,                    /*!< 27: USB SRAM block clock */
    SYSCTL_CLOCK_CRC,                       /*!< 25: CRC clock */
    SYSCTL_CLOCK_DMA,                       /*!< 25: DMA clock */
    SYSCTL_CLOCK_RTC,                       /*!< 25: RTC clock */
    SYSCTL_CLOCK_SCT0_1,                    /*!< 25: SCT 0/1 clock */
} CHIP_SYSCTL_CLOCK_T;





#endif /* __CLOCK_11U6X_H_ */
