/*
 * @brief LPC11U6X System Control registers and control functions
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __SYSCTL_11U6X_H_
#define __SYSCTL_11U6X_H_


/**
 * @brief LPC11U6X System Control block structure
 */
typedef struct {                        /*!< SYSCTL Structure */
    __IO uint32_t  SYSMEMREMAP;         /*!< System Memory remap register */
    __IO uint32_t  PRESETCTRL;          /*!< Peripheral reset Control register */
    __IO uint32_t  SYSPLLCTRL;          /*!< System PLL control register */
    __I  uint32_t  SYSPLLSTAT;          /*!< System PLL status register */
    __IO uint32_t  USBPLLCTRL;          /*!< USB PLL control register */
    __I  uint32_t  USBPLLSTAT;          /*!< USB PLL status register */
    __I  uint32_t  RESERVED1[1];
    __IO uint32_t  RTCOSCCTRL;          /*!< RTC Oscillator control register */
    __IO uint32_t  SYSOSCCTRL;          /*!< System Oscillator control register */
    __IO uint32_t  WDTOSCCTRL;          /*!< Watchdog Oscillator control register */
    __I  uint32_t  RESERVED2[2];
    __IO uint32_t  SYSRSTSTAT;          /*!< System Reset Status register */
    __I  uint32_t  RESERVED3[3];
    __IO uint32_t  SYSPLLCLKSEL;        /*!< System PLL clock source select register */
    __IO uint32_t  SYSPLLCLKUEN;        /*!< System PLL clock source update enable register*/
    __IO uint32_t  USBPLLCLKSEL;        /*!< USB PLL clock source select register */
    __IO uint32_t  USBPLLCLKUEN;        /*!< USB PLL clock source update enable register */
    __I  uint32_t  RESERVED4[8];
    __IO uint32_t  MAINCLKSEL;          /*!< Main clock source select register */
    __IO uint32_t  MAINCLKUEN;          /*!< Main clock source update enable register */
    __IO uint32_t  SYSAHBCLKDIV;        /*!< System Clock divider register */
    __I  uint32_t  RESERVED5;
    __IO uint32_t  SYSAHBCLKCTRL;       /*!< System clock control register */
    __I  uint32_t  RESERVED6[4];
    __IO uint32_t  SSP0CLKDIV;          /*!< SSP0 clock divider register */
    __IO uint32_t  USART0CLKDIV;        /*!< UART clock divider register */
    __IO uint32_t  SSP1CLKDIV;          /*!< SSP1 clock divider register */
    __IO uint32_t  FRGCLKDIV;           /*!< FRG clock divider (USARTS 1 - 4) register */
    __I  uint32_t  RESERVED7[7];
    __IO uint32_t  USBCLKSEL;           /*!< USB clock source select register */
    __IO uint32_t  USBCLKUEN;           /*!< USB clock source update enable register */
    __IO uint32_t  USBCLKDIV;           /*!< USB clock source divider register */
    __I  uint32_t  RESERVED8[5];
    __IO uint32_t  CLKOUTSEL;           /*!< Clock out source select register */
    __IO uint32_t  CLKOUTUEN;           /*!< Clock out source update enable register */
    __IO uint32_t  CLKOUTDIV;           /*!< Clock out divider register */
    __I  uint32_t  RESERVED9;
    __IO uint32_t  UARTFRGDIV;          /*!< USART fractional generator divider (USARTS 1 - 4) register */
    __IO uint32_t  UARTFRGMULT;         /*!< USART fractional generator multiplier (USARTS 1 - 4) register */
    __I  uint32_t  RESERVED10;
    __IO uint32_t  EXTTRACECMD;         /*!< External trace buffer command register */
    __I  uint32_t  PIOPORCAP[3];        /*!< POR captured PIO status registers */
    __I  uint32_t  RESERVED11[10];
    __IO uint32_t  IOCONCLKDIV[7];      /*!< IOCON block for programmable glitch filter divider registers */
    __IO uint32_t  BODCTRL;             /*!< Brown Out Detect register */
    __IO uint32_t  SYSTCKCAL;           /*!< System tick counter calibration register */
    __I  uint32_t  RESERVED12[6];
    __IO uint32_t  IRQLATENCY;          /*!< IRQ delay register */
    __IO uint32_t  NMISRC;              /*!< NMI source control register */
    __IO uint32_t  PINTSEL[8];          /*!< GPIO pin interrupt select register 0-7 */
    __IO uint32_t  USBCLKCTRL;          /*!< USB clock control register */
    __I  uint32_t  USBCLKST;            /*!< USB clock status register */
    __I  uint32_t  RESERVED13[25];
    __IO uint32_t  STARTERP0;           /*!< Start logic 0 interrupt wake-up enable register */
    __I  uint32_t  RESERVED14[3];
    __IO uint32_t  STARTERP1;           /*!< Start logic 1 interrupt wake-up enable register */
    __I  uint32_t  RESERVED15[6];
    __IO uint32_t  PDSLEEPCFG;          /*!< Power down states in deep sleep mode register */
    __IO uint32_t  PDWAKECFG;           /*!< Power down states in wake up from deep sleep register */
    __IO uint32_t  PDRUNCFG;            /*!< Power configuration register*/
    __I  uint32_t  RESERVED16[110];
    __I  uint32_t  DEVICEID;            /*!< Device ID register */
} LPC_SYSCTL_T;

/**
 * System memory remap modes used to remap interrupt vectors
 */
typedef enum CHIP_SYSCTL_BOOT_MODE_REMAP {
    REMAP_BOOT_LOADER_MODE,    /*!< Interrupt vectors are re-mapped to Boot ROM */
    REMAP_USER_RAM_MODE,    /*!< Interrupt vectors are re-mapped to Static RAM */
    REMAP_USER_FLASH_MODE    /*!< Interrupt vectors are not re-mapped and reside in Flash */
} CHIP_SYSCTL_BOOT_MODE_REMAP_T;


/**
 * Peripheral reset identifiers, not available on all devices
 */
typedef enum {
    RESET_SSP0,            /*!< SSP0 reset control */
    RESET_I2C0,            /*!< I2C0 reset control */
    RESET_SSP1,            /*!< SSP1 reset control */
    RESET_I2C1,            /*!< I2C1 reset control */
    RESET_FRG,            /*!< FRG reset control */
    RESET_USART1,        /*!< USART1 reset control */
    RESET_USART2,        /*!< USART1 reset control */
    RESET_USART3,        /*!< USART1 reset control */
    RESET_USART4,        /*!< USART1 reset control */
    RESET_SCT0,            /*!< SCT0 reset control */
    RESET_SCT1            /*!< SCT1 reset control */
} CHIP_SYSCTL_PERIPH_RESET_T;


/**
 * System reset status
 */
#define SYSCTL_RST_POR    (1 << 0)    /*!< POR reset status */
#define SYSCTL_RST_EXTRST (1 << 1)    /*!< External reset status */
#define SYSCTL_RST_WDT    (1 << 2)    /*!< Watchdog reset status */
#define SYSCTL_RST_BOD    (1 << 3)    /*!< Brown-out detect reset status */
#define SYSCTL_RST_SYSRST (1 << 4)    /*!< software system reset status */


/**
 * Brown-out detector reset level
 */
typedef enum CHIP_SYSCTL_BODRSTLVL {
    // FIXME - will update with correct voltages
    SYSCTL_BODRSTLVL_LEVEL0,    /*!< Brown-out reset at TBD volts */
    SYSCTL_BODRSTLVL_LEVEL1,    /*!< Brown-out reset at TBD volts */
    SYSCTL_BODRSTLVL_LEVEL2,    /*!< Brown-out reset at TBD volts */
    SYSCTL_BODRSTLVL_LEVEL3,    /*!< Brown-out reset at TBD volts */
} CHIP_SYSCTL_BODRSTLVL_T;

/**
 * Brown-out detector interrupt level
 */
typedef enum CHIP_SYSCTL_BODRINTVAL {
    // FIXME - will update with correct voltages
    SYSCTL_BODINTVAL_RESERVED1,
    SYSCTL_BODINTVAL_RESERVED2,
    SYSCTL_BODINTVAL_2_LEVEL2,    /*!< Brown-out interrupt at TBD volts */
    SYSCTL_BODINTVAL_2_LEVEL3,    /*!< Brown-out interrupt at TBD volts */
} CHIP_SYSCTL_BODRINTVAL_T;



/**
 * Peripheral interrupt wakeup events
 */
// FIXME - Need to recheck these bit positions
// FIXME - LPC11U6X user manual shows this as bit 13.  This is the bit that works!
#define SYSCTL_WAKEUP_BOD_WDT_INT (1 << 12)    /*!< Shared Brown Out Detect (BOD) and WDT interrupt wake-up */
// FIXME - LPC11U6X user manual shows this as bit 12.  This is the bit that works!
#define SYSCTL_WAKEUP_RTCINT     (1 << 19)    /*!< RTC interrupt wake-up */
#define SYSCTL_WAKEUP_USB_WAKEUP (1 << 19)    /*!< USB need_clock signal wake-up */
#define SYSCTL_WAKEUP_GPIOINT0   (1 << 20)    /*!< GPIO GROUP0 interrupt wake-up */
#define SYSCTL_WAKEUP_GPIOINT1   (1 << 21)    /*!< GPIO GROUP1 interrupt wake-up */
#define SYSCTL_WAKEUP_USART1_4   (1 << 23)    /*!< Combined USART1 and USART4 wake-up */
#define SYSCTL_WAKEUP_USART2_3   (1 << 24)    /*!< Combined USART2 and USART3 interrupt wake-up */


/**
 * Deep sleep setup values
 */
#define SYSCTL_DEEPSLP_BOD_PD    (1 << 3)    /*!< BOD power-down control in Deep-sleep mode, powered down */
#define SYSCTL_DEEPSLP_WDTOSC_PD (1 << 6)    /*!< Watchdog oscillator power control in Deep-sleep, powered down */


/**
 * Deep sleep to wakeup setup values
 */
#define SYSCTL_SLPWAKE_IRCOUT_PD    (1 << 0)    /*!< IRC oscillator output wake-up configuration */
#define SYSCTL_SLPWAKE_IRC_PD       (1 << 1)    /*!< IRC oscillator power-down wake-up configuration */
#define SYSCTL_SLPWAKE_FLASH_PD     (1 << 2)    /*!< Flash wake-up configuration */
#define SYSCTL_SLPWAKE_BOD_PD       (1 << 3)    /*!< BOD wake-up configuration */
#define SYSCTL_SLPWAKE_ADC_PD       (1 << 4)    /*!< ADC wake-up configuration */
#define SYSCTL_SLPWAKE_SYSOSC_PD    (1 << 5)    /*!< System oscillator wake-up configuration */
#define SYSCTL_SLPWAKE_WDTOSC_PD    (1 << 6)    /*!< Watchdog oscillator wake-up configuration */
#define SYSCTL_SLPWAKE_SYSPLL_PD    (1 << 7)    /*!< System PLL wake-up configuration */
#define SYSCTL_SLPWAKE_USBPLL_PD    (1 << 8)    /*!< USB PLL wake-up configuration */
#define SYSCTL_SLPWAKE_USBPAD_PD    (1 << 10)    /*!< USB transceiver wake-up configuration */
#define SYSCTL_SLPWAKE_TS_PD        (1 << 13)    /*!< Temperature sensor wake-up configuration */



/**
 * Power down configuration values
 */
#define SYSCTL_POWERDOWN_IRCOUT_PD (1 << 0)        /*!< IRC oscillator output power down */
#define SYSCTL_POWERDOWN_IRC_PD    (1 << 1)        /*!< IRC oscillator power-down */
#define SYSCTL_POWERDOWN_FLASH_PD  (1 << 2)        /*!< Flash power down */
#define SYSCTL_POWERDOWN_BOD_PD    (1 << 3)        /*!< BOD power down */
#define SYSCTL_POWERDOWN_ADC_PD    (1 << 4)        /*!< ADC power down */
#define SYSCTL_POWERDOWN_SYSOSC_PD (1 << 5)        /*!< System oscillator power down */
#define SYSCTL_POWERDOWN_WDTOSC_PD (1 << 6)        /*!< Watchdog oscillator power down */
#define SYSCTL_POWERDOWN_SYSPLL_PD (1 << 7)        /*!< System PLL power down */
#define SYSCTL_POWERDOWN_USBPLL_PD (1 << 8)        /*!< USB PLL power-down */
#define SYSCTL_POWERDOWN_USBPAD_PD (1 << 10)    /*!< USB transceiver power-down */
#define SYSCTL_POWERDOWN_TS_PD     (1 << 13)    /*!< Temperature Sensor power-down */


#endif /*!< __SYSCTL_11U6X_H_ */
