/*
 * @brief LPC11u6xx USART1/2/3/4 driver
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __UART_N_11U6X_H_
#define __UART_N_11U6X_H_



/**
 * @brief UART register block structure
 */
typedef struct {
    __IO uint32_t  CFG;                /*!< Configuration register */
    __IO uint32_t  CTRL;            /*!< Control register */
    __IO uint32_t  STAT;            /*!< Status register */
    __IO uint32_t  INTENSET;        /*!< Interrupt Enable read and set register */
    __O  uint32_t  INTENCLR;        /*!< Interrupt Enable clear register */
    __I  uint32_t  RXDATA;            /*!< Receive Data register */
    __I  uint32_t  RXDATA_STAT;        /*!< Receive Data with status register */
    __IO uint32_t  TXDATA;            /*!< Transmit data register */
    __IO uint32_t  BRG;                /*!< Baud Rate Generator register */
    __IO uint32_t  INTSTAT;            /*!< Interrupt status register */
    __IO uint32_t  OSR;                /*!< Oversample selection register for asynchronous communication */
    __IO uint32_t  ADDR;            /*!< Address register for automatic address matching */
} LPC_USARTN_T;

/**
 * @brief UART CFG register definitions
 */
#define UARTN_CFG_ENABLE         (0x01 << 0)
#define UARTN_CFG_DATALEN_7      (0x00 << 2)    /*!< UART 7 bit length mode */
#define UARTN_CFG_DATALEN_8      (0x01 << 2)    /*!< UART 8 bit length mode */
#define UARTN_CFG_DATALEN_9      (0x02 << 2)    /*!< UART 9 bit length mode */
#define UARTN_CFG_PARITY_NONE    (0x00 << 4)    /*!< No parity */
#define UARTN_CFG_PARITY_EVEN    (0x02 << 4)    /*!< Even parity */
#define UARTN_CFG_PARITY_ODD     (0x03 << 4)    /*!< Odd parity */
#define UARTN_CFG_STOPLEN_1      (0x00 << 6)    /*!< UART One Stop Bit Select */
#define UARTN_CFG_STOPLEN_2      (0x01 << 6)    /*!< UART Two Stop Bits Select */
#define UARTN_MODE_32K           (0x01 << 7)    /*!< Selects the 32 kHz clock from the RTC oscillator as the clock source to the BRG */
#define UARTN_CFG_CTSEN          (0x01 << 9)    /*!< CTS enable bit */
#define UARTN_CFG_SYNCEN         (0x01 << 11)    /*!< Synchronous mode enable bit */
#define UARTN_CFG_CLKPOL         (0x01 << 12)    /*!< Un_RXD rising edge sample enable bit */
#define UARTN_CFG_SYNCMST        (0x01 << 14)    /*!< Select master mode (synchronous mode) enable bit */
#define UARTN_CFG_LOOP           (0x01 << 15)    /*!< Loopback mode enable bit */

/**
 * @brief UART CTRL register definitions
 */
#define UARTN_CTRL_TXBRKEN       (0x01 << 1)        /*!< Continuous break enable bit */
#define UARTN_CTRL_ADDRDET       (0x01 << 2)        /*!< Address detect mode enable bit */
#define UARTN_CTRL_TXDIS         (0x01 << 6)        /*!< Transmit disable bit */
#define UARTN_CTRL_CC            (0x01 << 8)        /*!< Continuous Clock mode enable bit */
#define UARTN_CTRL_CLRCC         (0x01 << 9)        /*!< Clear Continuous Clock bit */

/**
 * @brief UART STAT register definitions
 */
#define UARTN_STAT_RXRDY         (0x01 << 0)            /*!< Receiver ready */
#define UARTN_STAT_RXIDLE        (0x01 << 1)            /*!< Receiver idle */
#define UARTN_STAT_TXRDY         (0x01 << 2)            /*!< Transmitter ready for data */
#define UARTN_STAT_TXIDLE        (0x01 << 3)            /*!< Transmitter idle */
#define UARTN_STAT_CTS           (0x01 << 4)            /*!< Status of CTS signal */
#define UARTN_STAT_DELTACTS      (0x01 << 5)            /*!< Change in CTS state */
#define UARTN_STAT_TXDISINT      (0x01 << 6)            /*!< Transmitter disabled */
#define UARTN_STAT_OVERRUNINT    (0x01 << 8)            /*!< Overrun Error interrupt flag. */
#define UARTN_STAT_RXBRK         (0x01 << 10)        /*!< Received break */
#define UARTN_STAT_DELTARXBRK    (0x01 << 11)        /*!< Change in receive break detection */
#define UARTN_STAT_START         (0x01 << 12)        /*!< Start detected */
#define UARTN_STAT_FRM_ERRINT    (0x01 << 13)        /*!< Framing Error interrupt flag */
#define UARTN_STAT_PAR_ERRINT    (0x01 << 14)        /*!< Parity Error interrupt flag */
#define UARTN_STAT_RXNOISEINT    (0x01 << 15)        /*!< Received Noise interrupt flag */

/**
 * @brief UART INTENSET/INTENCLR register definitions
 */
#define UARTN_INTEN_RXRDY        (0x01 << 0)            /*!< Receive Ready interrupt */
#define UARTN_INTEN_TXRDY        (0x01 << 2)            /*!< Transmit Ready interrupt */
#define UARTN_INTEN_DELTACTS     (0x01 << 5)            /*!< Change in CTS state interrupt */
#define UARTN_INTEN_TXDIS        (0x01 << 6)            /*!< Transmitter disable interrupt */
#define UARTN_INTEN_OVERRUN      (0x01 << 8)            /*!< Overrun error interrupt */
#define UARTN_INTEN_DELTARXBRK   (0x01 << 11)        /*!< Change in receiver break detection interrupt */
#define UARTN_INTEN_START        (0x01 << 12)        /*!< Start detect interrupt */
#define UARTN_INTEN_FRAMERR      (0x01 << 13)        /*!< Frame error interrupt */
#define UARTN_INTEN_PARITYERR    (0x01 << 14)        /*!< Parity error interrupt */
#define UARTN_INTEN_RXNOISE      (0x01 << 15)        /*!< Received noise interrupt */


#endif /* __UART_N_11U6X_H_ */
