/*
 * @brief LPC18xx/43xx Pin Interrupt and Pattern Match Registers and driver
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __PININT_18XX_43XX_H_
#define __PININT_18XX_43XX_H_

/**
 * @brief LPC18xx/43xx Pin Interrupt and Pattern Match register block structure
 */
typedef struct {            /*!< PIN_INT Structure */
    __IO uint32_t ISEL;     /*!< Pin Interrupt Mode register */
    __IO uint32_t IENR;     /*!< Pin Interrupt Enable (Rising) register */
    __IO uint32_t SIENR;    /*!< Set Pin Interrupt Enable (Rising) register */
    __IO uint32_t CIENR;    /*!< Clear Pin Interrupt Enable (Rising) register */
    __IO uint32_t IENF;     /*!< Pin Interrupt Enable Falling Edge / Active Level register */
    __IO uint32_t SIENF;    /*!< Set Pin Interrupt Enable Falling Edge / Active Level register */
    __IO uint32_t CIENF;    /*!< Clear Pin Interrupt Enable Falling Edge / Active Level address */
    __IO uint32_t RISE;     /*!< Pin Interrupt Rising Edge register */
    __IO uint32_t FALL;     /*!< Pin Interrupt Falling Edge register */
    __IO uint32_t IST;      /*!< Pin Interrupt Status register */
} LPC_PIN_INT_T;


/**
 * LPC18xx/43xx Pin Interrupt channel values
 */
#define PININTCH0         (1 << 0)
#define PININTCH1         (1 << 1)
#define PININTCH2         (1 << 2)
#define PININTCH3         (1 << 3)
#define PININTCH4         (1 << 4)
#define PININTCH5         (1 << 5)
#define PININTCH6         (1 << 6)
#define PININTCH7         (1 << 7)
#define PININTCH(ch)      (1 << (ch))

#endif /* __PININT_18XX_43XX_H_ */
