# makefile для включения в проект.
#
# на входе принимет параметры:
#  TARGET_CPU (или может быть переопределен через LIP_TARGET) - определяет,
#              какой порт будет использоваться. Должен быть задан один
#              из поддерживаемых портов
#  LIP_DIR  - директория корня исходных кодов lip относительно файла проекта
#              (по-умолчанию lib/lip)
#
# устанавливает следующие параметры:
#  LIP_SRC         - исходные файлы на C
#  LIP_HDR         - заголовочные файлы lip (обычно не используются проектом)
#  LIP_INC_DIRS    - директории с заголовками, которые должны быть добавлены
#                    в список директорий для поиска заголовков проекта
#
# также создается цель $(LIP_DOC) для сборки документации и lip_clean для ее очистки


LIP_DIR ?= ./lib/lip
LIP_TARGET ?= $(TARGET_CPU)

#lip sources
LIP_SRC = $(LIP_DIR)/src/phy/lip_phy.c \
          $(LIP_DIR)/src/phy/dp83848j/lip_phy_dp83848j.c \
          $(LIP_DIR)/src/phy/lan8720/lip_phy_lan8720.c \
          $(LIP_DIR)/src/lip_core.c \
          $(LIP_DIR)/src/misc/lip_rand.c \
          $(LIP_DIR)/src/link/lip_eth.c \
          $(LIP_DIR)/src/link/lip_arp.c \
          $(LIP_DIR)/src/net/ipv4/lip_ip.c \
          $(LIP_DIR)/src/net/ipv4/lip_ip_link_local.c \
          $(LIP_DIR)/src/net/ipv4/lip_ipv4_acd.c \
          $(LIP_DIR)/src/net/ipv4/lip_ip_reassembly.c \
          $(LIP_DIR)/src/net/ipv4/lip_icmp.c \
          $(LIP_DIR)/src/net/ipv4/lip_igmp.c \
          $(LIP_DIR)/src/transport/udp/lip_udp.c \
          $(LIP_DIR)/src/transport/tcp/lip_tcp.c \
          $(LIP_DIR)/src/app/dhcp/lip_dhcpc.c \
          $(LIP_DIR)/src/app/tftp/lip_tftp.c \
          $(LIP_DIR)/src/app/dns/mdns/lip_mdns.c \
          $(LIP_DIR)/src/app/dns/dns_sd/lip_dns_sd.c \
          $(LIP_DIR)/src/app/modbus/lip_mb_tcp_server.c \
          $(LIP_DIR)/src/app/modbus/core/lip_mb_server.c

#lip headers
LIP_HDR = $(LIP_DIR)/src/conf/lip_conf.h \
          $(LIP_DIR)/src/phy/lip_phy.h \
          $(LIP_DIR)/src/phy/phy_regs.h \
          $(LIP_DIR)/src/lip.h \
          $(LIP_DIR)/src/lip_defs.h \
          $(LIP_DIR)/src/lip_ll.h \
          $(LIP_DIR)/src/lip_log.h \
          $(LIP_DIR)/src/lip_private.h \
          $(LIP_DIR)/src/misc/lip_rand.h \
          $(LIP_DIR)/src/link/lip_eth.h \
          $(LIP_DIR)/src/link/lip_arp.h \
          $(LIP_DIR)/src/net/lip_ip.h \
          $(LIP_DIR)/src/net/ipv4/lip_ipv4.h \
          $(LIP_DIR)/src/net/ipv4/lip_ip_private.h \
          $(LIP_DIR)/src/net/ipv4/lip_ip_link_local.h \
          $(LIP_DIR)/src/net/ipv4/lip_ip_link_local_private.h \
          $(LIP_DIR)/src/net/ipv4/lip_ipv4_acd.h \
          $(LIP_DIR)/src/net/ipv4/lip_ip_reassembly_private.h \
          $(LIP_DIR)/src/net/ipv4/lip_icmp.h \
          $(LIP_DIR)/src/net/ipv4/lip_igmp.h \
          $(LIP_DIR)/src/transport/udp/lip_udp_private.h \
          $(LIP_DIR)/src/transport/udp/lip_udp.h \
          $(LIP_DIR)/src/transport/tcp/lip_tcp_private.h \
          $(LIP_DIR)/src/transport/tcp/lip_tcp.h \
          $(LIP_DIR)/src/transport/tcp/lip_sock.h \
          $(LIP_DIR)/src/app/dhcp/lip_dhcp_const.h \
          $(LIP_DIR)/src/app/dhcp/lip_dhcpc_private.h \
          $(LIP_DIR)/src/app/dhcp/lip_dhcpc.h \
          $(LIP_DIR)/src/app/tftp/lip_tftp_private.h \
          $(LIP_DIR)/src/app/tftp/lip_tftp.h \
          $(LIP_DIR)/src/app/dns/lip_dns_defs.h \
          $(LIP_DIR)/src/app/dns/mdns/lip_mdns_private.h \
          $(LIP_DIR)/src/app/dns/mdns/lip_mdns.h \
          $(LIP_DIR)/src/app/dns/dns_sd/lip_dns_sd.h \
          $(LIP_DIR)/src/app/dns/dns_sd/lip_dns_sd_private.h \
          $(LIP_DIR)/src/app/modbus/lip_mb_tcp_server.h \
          $(LIP_DIR)/src/app/modbus/lip_mb_tcp_defs.h \
          $(LIP_DIR)/src/app/modbus/core/lip_mb_server.h \
          $(LIP_DIR)/src/app/modbus/core/lip_mb_defs.h
          
#include path
LIP_INC_DIRS = $(LIP_DIR)/src
LIP_INCDIRS  = $(LIP_INC_DIRS)

#documentation files
LIP_DOC = $(LIP_DIR)/doc/html/index.html

#append port file to sources
ifeq ($(LIP_TARGET), lpc17xx)
    LIP_SRC += $(LIP_DIR)/src/ports/lpc17xx/lip_ll_lpc17xx.c
else ifeq ($(LIP_TARGET), lpc43xx)
    LIP_SRC += $(LIP_DIR)/src/ports/lpc43xx/lip_ll_lpc43xx.c
    LIP_SRC += $(LIP_DIR)/src/ports/lpc43xx/lip_ll_lpc43xx_descr.c
else ifeq ($(LIP_TARGET), bf609)
    LIP_SRC += $(LIP_DIR)/src/ports/bf609/lip_ll_bf609.c
    LIP_SRC += $(LIP_DIR)/src/ports/lpc43xx/lip_ll_lpc43xx_descr.c
else
    $(warning lip warn: port architecture should be specified in LIP_TARGET variable)
endif

ifdef LIP_ENABLE_DOC
#target for create lip doxygen documentation
$(LIP_DOC): $(LIP_SRC) $(LIP_HDR) $(LIP_DIR)/doc/Doxyfile $(LIP_DIR)/doc/mainpage.md
	doxygen $(LIP_DIR)/doc/Doxyfile

lip_clean:
	-rm -fR $(LIP_DIR)/doc/html
	-rm -fR $(LIP_DIR)/doc/rtf
endif
