#include "lip.h"

#include "lip_ll.h"
#include "phy/lip_phy.h"
#include "lip_sample_phy_loop.h"

int f_init_done = 0;
int f_init_flags = LIP_INIT_FLAGS_LOOPBACK | LIP_INIT_FLAGS_RX_FILTER_DIS;
static uint32_t f_tx_cntr = 0;
static uint32_t f_rx_cntr = 0;


void lip_sample_phy_loop_init(void) {
    /* инициализация стека */
    f_init_done = 0;
    f_tx_cntr = f_rx_cntr = 0;
    lip_phy_init_start(f_init_flags);
}

int lip_sample_phy_loop_pull(t_lip_sample_phy_loop_stat *stat) {
    int err = 0;
    int out = 0;
    if (!f_init_done) {
        int init_res = lip_phy_init_pull(f_init_flags);
        if (init_res  == LIP_ERR_SUCCESS) {
            f_init_done = 1;
        } else if (init_res != LIP_ERR_INIT_IN_PROGRESS) {
            /* не удалось инициализировать phy */
            err = init_res;
        }
    }

    if (f_init_done) {
        int max_size = 0;
        size_t rx_size;

        uint32_t *rx_buf;

        /* сперва вычитываем все принятые пакеты и проверяем на счетчик */
        while (!out && ((rx_buf = (uint32_t*)lip_ll_rx_get_buf(&rx_size)))!=NULL) {
            size_t i;
            stat->rcv_packts++;

            for (i=0; i < rx_size/4; i++) {
                if (rx_buf[i]!=f_rx_cntr) {
                    stat->last_err.exp_wrd = f_rx_cntr;
                    stat->last_err.rcv_wrd = rx_buf[i];
                    stat->rcv_errs++;
                    f_rx_cntr = rx_buf[rx_size/4 - 1];
                    out = 1;
                    break;
                }
                f_rx_cntr++;
            }
            stat->rcv_cntr = f_rx_cntr;
            lip_ll_rx_drop();
        }

        /* затем посылаем пакет, если есть место */
        uint32_t *tx_buf = (uint32_t*)lip_ll_tx_get_buf(&max_size);
        if (tx_buf!=NULL) {
            int snd_size = 0, i;
            for (i=0; i < max_size/4; i++) {
                tx_buf[i] = f_tx_cntr++;
                snd_size+=4;
            }
            /* передача сформированного пакета */
            lip_ll_tx_flush(snd_size);
        }
    }
    return err;
}


void lip_sample_phy_loop_finish(void) {
    lip_ll_close();
#ifdef LIP_PHY_RESET_SET
    /* перевод Phy в состояние сброса (если макрос определен) */
    LIP_PHY_RESET_SET();
#endif
}
