/* Пример реализации кольца на уровне phy.

   Данный тест не предполагает штатной работы стека и требует, чтобы он был
   закрыт lip_close(), если был инициализирован ранее.

   Для работы нужно вызвать lip_sample_phy_loop_init() и потом периодически
   вызывать lip_sample_phy_loop_pull().
   По завершению работы нужно вызвать lip_sample_phy_loop_finish(), после чего
   PHY находится в отключенном состоянии (и для дальнейшей штатной работы
   стека нужно будет вызвать снова lip_init() с последующими операциями).

   Пример работает напрямую с функциями порта под контроллер, биз использования
   самого стека. Пример передает пакеты с счетчиком с максимально возможной
   скоростью и при приеме анализирует целостность принятых пакетов.

   Счетчик принятых пакетов и ошибок обновляется в переданной структуре со
   статистикой.
*/

#ifndef LIP_SAMPLE_PHY_LOOP_H
#define LIP_SAMPLE_PHY_LOOP_H

/** Структура со статистикой кольцевого теста */
typedef struct {
    uint32_t rcv_packts; /**< Количество принятых пакетов */
    uint32_t rcv_cntr;   /**< Счетчик принятых слов */
    uint32_t rcv_errs;   /**< Количество ошибок */
    struct {
        uint32_t rcv_wrd; /**< Слово, которое было принято */
        uint32_t exp_wrd; /**< Слово, которое ожидали */
    } last_err; /**< Информация о последней ошибке */
} t_lip_sample_phy_loop_stat;

void lip_sample_phy_loop_init(void);
int lip_sample_phy_loop_pull(t_lip_sample_phy_loop_stat *stat);
void lip_sample_phy_loop_finish(void);

#endif // LIP_SAMPLE_PHY_LOOP_H
