/*
 * lip_eth.c
 *
 *  Created on: 31.08.2010
 *      Author: borisov
 */

#ifndef LIP_ETH_C_
#define LIP_ETH_C_

#include "lip_defs.h"
#include "lcspec.h"

#define LIP_ETH_FLAGS_BROADCAST         0x2
#define LIP_ETH_FLAGS_MULTICAST         0x4

/** Размер MAC-адреса */
#define LIP_MAC_ADDR_SIZE    6

/** Максимальный размер кадра при пакета по Ethernet */
#define LIP_ETH_MAX_PACKET_SIZE  1514

/* тип протокола, используется как в пакете
 * Ethernet, так и в пакете ARP
 */
typedef enum {
    e_LIP_ETHER_TYPE_IPv4 = 0x0800,
    e_LIP_ETHER_TYPE_ARP  = 0x0806,
    e_LIP_ETHER_TYPE_IPv6 = 0x86dd
} t_lip_ether_type;


#include "lcspec_pack_start.h"
struct st_lip_eth_hdr {
    uint8_t da[LIP_MAC_ADDR_SIZE];
    uint8_t sa[LIP_MAC_ADDR_SIZE];
    uint16_t type;
} LATTRIBUTE_PACKED;
typedef struct st_lip_eth_hdr t_lip_eth_hdr;
#include "lcspec_pack_restore.h"

#define LIP_ETH_HDR_SIZE (sizeof(struct st_lip_eth_hdr))



extern const uint8_t g_lip_broadcast_mac[LIP_MAC_ADDR_SIZE];

#define lip_eth_is_bcast(_addr) !memcmp(_addr, g_lip_broadcast_mac, LIP_MAC_ADDR_SIZE)

/***************************************************************************//** 
    Установка mac-адреса устройства. Функция должна вызываться сразу после
    вызова lip_init() 
    @param mac   указатель на массив длиной #LIP_MAC_ADDR_SIZE с устанавливаемым 
                 MAC-адресом 
 ******************************************************************************/
void lip_eth_set_mac_addr(const uint8_t *mac);

const uint8_t *lip_eth_cur_mac(void);

t_lip_errs lip_eth_send(const uint8_t *buf, size_t size, t_lip_ether_type type, const uint8_t *dest_addr);
void* lip_eth_prepare(t_lip_ether_type type, const uint8_t *dest_addr, int *max_size);
t_lip_errs lip_eth_flush_split(size_t size, const uint8_t* data, int data_size);
#define lip_eth_flush(size) lip_eth_flush_split(size, NULL, 0)


#endif /* LIP_ETH_C_ */
