/*
 * lip_ip_private.h
 *
 *  Created on: 05.08.2011
 *      Author: Borisov
 */

#ifndef LIP_IP_PRIVATE_H_
#define LIP_IP_PRIVATE_H_

#include "net/lip_ip.h"
#include "lcspec.h"

//коды протоколов верхнего уровня
#define LIP_PROTOCOL_ICMP  1
#define LIP_PROTOCOL_IGMP  2
#define LIP_PROTOCOL_TCP   6
#define LIP_PROTOCOL_UDP   17
#define LIP_PROTOCOL_ICMP6 58


#define LIP_IP_FLAG_UNSPEC_LOCAL_IP     0x1
#define LIP_IP_FLAG_BROADCAST           0x2
#define LIP_IP_FLAG_MULTICAST           0x4
//указывает, что нужно добавить соответствующую опции при посылке (rfc2113)
#define LIP_IP_FLAG_ROUTER_ALERT        0x8
//признак, что у принятой дейтаграммы неверный ip-адрес
#define LIP_IP_FLAG_INVALID_DEST_ADDR   0x10
//признак, что дейтаграмма не целая, а фрагмент
#define LIP_IP_FLAG_FRAGMETED           0x20

/***************  флаги, задающие Type Of Service ****************/
//значения поля precedence
#define LIP_IP_PRECEDENCE_NETWORK_CONTROL       0xE0
#define LIP_IP_PRECEDENCE_INTERNETWORK_CONTROL  0xC0
#define LIP_IP_PRECEDENCE_CRITIC_ECP            0xA0
#define LIP_IP_PRECEDENCE_FLASH_OVERRIDE        0x80
#define LIP_IP_PRECEDENCE_FLASH                 0x60
#define LIP_IP_PRECEDENCE_IMMEDIATE             0x40
#define LIP_IP_PRECEDENCE_PRIORITY              0x20
#define LIP_IP_PRECEDENCE_ROUTINE               0x00
//доп. флаги
#define LIP_IP_TOS_LOW_DELAY                    0x10
#define LIP_IP_TOS_HIGH_TROUGHPUT               0x08
#define LIP_IP_TOS_HIGH_RELIBILITY              0x04


//максимальный размер заголовка IPv4
#define LIP_IPV4_MAX_HDR_SIZE                     60

/*****************************************************************
 * структура содержит полезную информацию из заголовка IP-пакета,
 * которую могут использовать протоколы верхнего уровня
 *****************************************************************/
typedef struct {
    uint32_t flags;
    const uint8_t* sa; //указатель на ip-адрес источника пакета
    const uint8_t* da; //указатель на ip-адрес назначения пакета    
    t_lip_ip_type type; //тип протокола IP (для различия IPv4, IPv6)
    uint8_t addr_len;  //длина ip-адреса
    uint8_t protocol;  //код протокола верхнего уровня
    uint8_t hdr_size;  //размер заголовка IP
    uint8_t ttl;
    uint8_t tos; //type of service
} t_lip_ip_info;





#include "lcspec_pack_start.h"
//заголовок IPv4
struct st_lip_ipv4_hdr
{
    uint8_t ver_ihl; //version | internet header length
    uint8_t tos; //Type of Service
    uint16_t length; //Total Length
    uint16_t id;    
    uint16_t offs_flgs; // Flags (3) + Fragment Offset (13)
    uint8_t time_to_live;
    uint8_t protocol;
    uint16_t checksum;
    uint8_t src_addr[LIP_IPV4_ADDR_SIZE];
    uint8_t dest_addr[LIP_IPV4_ADDR_SIZE];
    uint8_t opt[0];
} LATTRIBUTE_PACKED;
typedef struct st_lip_ipv4_hdr t_lip_ipv4_hdr;

#define LIP_IPV4_MIN_HDR_SIZE sizeof(struct st_lip_ipv4_hdr)


#include "lcspec_pack_restore.h"




void lip_ip_init(void);
void lip_ip_pull(void);
int lip_ip_process_packet(const uint8_t *data, size_t size, int ll_flags, t_lip_ip_info *info, uint16_t *data_size);

uint8_t* lip_ip_prepare(t_lip_ip_info *tx_info, int *max_size);

t_lip_errs lip_ip_flush_split(int size, const uint8_t* data, int data_size);
/***************************************************************************//**
 *  Передача подготовленного IP-пакета.
 *  Должна вызываться только после успешного вызова lip_ip_prepare() 
 *      до вызова lip_pull()
 *  @param[in]size  Размер данных для передачи протокола верхнего уровня
 *  @return         Код ошибки
 ******************************************************************************/
#define lip_ip_flush(size) lip_ip_flush_split(size, NULL, 0)



uint16_t lip_chksum(uint16_t sum, const uint8_t *data, int len);
int lip_ipv4_get_mcast_mac(const uint8_t* ip, uint8_t* mac);

void lip_ip_arp_resolve_cb(t_lip_ip_type ip_type, const uint8_t* mac, const uint8_t* ip_addr, uint32_t len);



#endif /* LIP_IP_PRIVATE_H_ */
