#include "lip_conf.h"
#ifdef LIP_PHY_SUPPORT_LAN8720

#include "phy/lip_phy_port.h"
#include "lan8720_regs.h"


static t_lip_errs f_get_mode_lan8720(uint8_t devaddr, e_LIP_PHY_SPEED *speed,
                                     e_LIP_PHY_MODE *mode);

t_lip_phy_info lip_phy_lan8720 = {
    "LAN8720",
    0x0007,
    0xC0F0,
    0xFFF0,
    NULL,
    f_get_mode_lan8720
};


static t_lip_errs f_get_mode_lan8720(uint8_t devaddr, e_LIP_PHY_SPEED *speed,
                                     e_LIP_PHY_MODE *mode) {
    t_lip_errs err = LIP_ERR_SUCCESS;
    uint16_t phy_reg;

    err = lip_phy_reg_read( devaddr, LAN8720_REG_PHYSCSR, &phy_reg );
    if (err == LIP_ERR_SUCCESS) {
        switch (phy_reg & LAN8720_BIT_PHYSCSR_SPEED_IND) {
            case 0x04:  // 10 BASE T Half-duplex
                *mode = LIP_PHY_MODE_HALF_DUPLEX;
                *speed = LIP_PHY_SPEED_10;
                break;
            case 0x08:  // 100 BASE TX Half-duplex
                *mode = LIP_PHY_MODE_HALF_DUPLEX;
                *speed = LIP_PHY_SPEED_100;
                break;
            case 0x14:  // 10 BASE T Full-duplex
                *mode = LIP_PHY_MODE_FULL_DUPLEX;
                *speed = LIP_PHY_SPEED_10;
                break;
            case 0x18:  // 100 BASE TX Full-duplex
                *mode = LIP_PHY_MODE_FULL_DUPLEX;
                *speed = LIP_PHY_SPEED_100;
                break;
            default:
                err = LIP_ERR_INIT_PHY_MODE;
                break;
        }
    }
    return err;

}

#endif
