/** @file lip_phy_port.h

   Файл содержит определение типов, необходимых для реализации поддержки
   конкретного PHY */

#ifndef LIP_PHY_PORT_H
#define LIP_PHY_PORT_H

#include "lip_defs.h"
#include "phy/lip_phy.h"


typedef t_lip_errs (* t_lip_phy_init)(uint8_t devaddr, int flags);

typedef t_lip_errs (* t_lip_phy_get_mode)(uint8_t devaddr, e_LIP_PHY_SPEED *speed,
                                          e_LIP_PHY_MODE *mode);


/** Структура описывает параметры поддерживаемых PHY-устройтсв. */
typedef struct {
    const char* name; /**< Название PHY (используется только для отладки) */
    uint16_t id1;     /**< Содержимое регистра PHYIDR1 */
    uint16_t id2;     /**< Содержимое регистра PHYIDR2 */
    uint16_t id2_msk; /**< Маска битов для проверки PHYIDR2 (проверяются только
                           установленные в 1 биты) */
    t_lip_phy_init        init; /**< Функция инициализации vendor-specific регистров,
                                     или NULL, если не требуется */
    t_lip_phy_get_mode get_mode; /**< Функция получения режима PHY,
                                         возвращает реально получившиеся значения
                                         скорости и режима PHY */

} t_lip_phy_info;

#endif // LIP_PHY_PORT_H
