/* Данный файл содержит определение общих (Base и Extended) регистров для
 * всех Phy-устройств. Определения Device-Specific регистров находятся
 * в phy/<название phy>/<название phy>_regs.h для каждого поддерживаемого PHY
 */

#ifndef PHY_REGS_H_
#define PHY_REGS_H_


#define LIP_PHY_REG_BMCR        0x00 /* Basic Mode Control Register */
#define LIP_PHY_REG_BMSR        0x01 /* Basic Mode Status Register */
#define LIP_PHY_REG_PHYIDR1     0x02 /* PHY Identifier Register #1 */
#define LIP_PHY_REG_PHYIDR2     0x03 /* PHY Identifier Register #2 */
#define LIP_PHY_REG_ANAR        0x04 /* Auto-Negotiation Advertisement */
#define LIP_PHY_REG_ANLPAR      0x05 /* Auto-Negotiation Link Partner Ability */
#define LIP_PHY_REG_ANER        0x06 /* Auto-Negotiate Expansion */
#define LIP_PHY_REG_ANNPTR      0x07 /* Auto-Negotiation Next Page Transmit */



/* BMCR bitmap */
#define BMCR_RESET              0x8000
#define BMCR_LOOPBACK           0x4000
#define BMCR_SPEED_100          0x2000
#define BMCR_AN                 0x1000
#define BMCR_POWERDOWN          0x0800
#define BMCR_ISOLATE            0x0400
#define BMCR_RE_AN              0x0200
#define BMCR_DUPLEX             0x0100

/* BMSR bitmap */
#define BMSR_100BE_T4           0x8000
#define BMSR_100TX_FULL         0x4000
#define BMSR_100TX_HALF         0x2000
#define BMSR_10BE_FULL          0x1000
#define BMSR_10BE_HALF          0x0800
#define BMSR_NOPREAM            0x0040
#define BMSR_AUTO_DONE          0x0020
#define BMSR_REMOTE_FAULT       0x0010
#define BMSR_NO_AUTO            0x0008
#define BMSR_LINK_ESTABLISHED   0x0004

/* PHY_ANAR bitmap */
#define ANAR_NEXT_PAGE          0x8000
#define ANAR_REMOTE_FAULT       0x2000
#define ANAR_PAUSE              0x0400
#define ANAR_100BE_T4           0x0200
#define ANAR_100BT_FULL         0x0100
#define ANAR_100BT              0x0080
#define ANAR_10BT_FULL          0x0040
#define ANAR_10BT               0x0020
#define ANAR_SELECTOR           0x001F

/* ANLPAR bitmap */
#define ANLPAR_NEXT_PAGE        0x8000
#define ANLPAR_ACKN             0x4000
#define ANLPAR_REMOTE_FAULT     0x2000
#define ANLPAR_PAUSE            0x0C00
#define ANLPAR_100BE_T4         0x0200
#define ANLPAR_100BT_FULL       0x0100
#define ANLPAR_100BT            0x0080
#define ANLPAR_10BT_FULL        0x0040
#define ANLPAR_10BT             0x0020
#define ANLPAR_SELECTOR         0x001F






#endif /* PHY_REGS_H_ */
