#include "lprintf.h"
#include "chip.h"
#include <stdlib.h>

volatile uint32_t *regStat = NULL;
volatile uint32_t *regTx   = NULL;
volatile uint32_t *regCtl  = NULL;


int lprintf_uart_init(unsigned port, unsigned baud, unsigned databits,
                      t_lprintf_uart_parity parity, unsigned tx_pin, unsigned flags) {
    int err = 0;
    unsigned ctl = 0;
    volatile uint32_t *regClk = NULL;

    /* unused params */
    (void)tx_pin;
    (void)flags;


    if (port == 0)     {
        regClk  = pREG_UART0_CLK;
        regCtl  = pREG_UART0_CTL;
        regStat = pREG_UART0_STAT;
        regTx   = pREG_UART0_THR;

        CHIP_PIN_CONFIG(CHIP_PIN_PD07_UART0_TX);
    } else if (port == 1) {
        regClk  = pREG_UART1_CLK;
        regCtl  = pREG_UART1_CTL;
        regStat = pREG_UART1_STAT;
        regTx   = pREG_UART1_THR;

        CHIP_PIN_CONFIG(CHIP_PIN_PG15_UART1_TX);
    } else {
        err = -1;
    }

    if (!err) {
        if (parity == LPRINTF_UART_PARITY_EVEN) {
            ctl |= (BITM_UART_CTL_EPS | BITM_UART_CTL_PEN);
        } else if (parity == LPRINTF_UART_PARITY_ODD) {
            ctl |= BITM_UART_CTL_PEN;
        } else if (parity != LPRINTF_UART_PARITY_NONE) {
            err = -1;
        }
    }

    if (!err) {
       if ((databits >= 5) && (databits <= 8)) {
           ctl |= (databits-5) << BITP_UART_CTL_WLS;
       } else {
           err = -1;
       }
    }

    if (!err) {
        unsigned baud_div = CHIP_CLKF_SCLK0/baud + 0.5;
        *regCtl = 0;
        *regClk = BITM_UART_CLK_EDBO | ((baud_div << BITP_UART_CLK_DIV) & BITM_UART_CLK_DIV);
        *regCtl = ctl | BITM_UART_CTL_EN;
    }


    return err;
}

void lprintf_putchar(char c) {
    while (!(*regStat & BITM_UART_STAT_THRE)) {}
    *regTx = c;
}

void lprintf_close(void) {
    if (regStat != NULL) {
        while (!(*regStat & BITM_UART_STAT_TEMT)) {}
        *regCtl = 0;
        if (regStat == pREG_UART0_STAT) {
            CHIP_PIN_CONFIG(CHIP_PIN_PD07_GPIO);
        } else {
            CHIP_PIN_CONFIG(CHIP_PIN_PG15_GPIO);
        }
        regStat = regCtl = regTx = NULL;
    }
}
