#include "chip.h"
#include "lprintf.h"


#define UART_FIFO_SIZE 16

unsigned f_tx_pin;
unsigned f_port;




#include "lpc_uart_baudrate.inc"


int lprintf_uart_init(unsigned port, unsigned baud, unsigned databits,
                      t_lprintf_uart_parity parity, unsigned tx_pin, unsigned flags) {

    int err = 0;

    if (port==0) {
        unsigned cfg = 0;
        LPC_CLK_EN(SYSCTL_CLOCK_UART0);

        switch (databits) {
            case 5:
                cfg |= UART0_LCR_WLEN5;
                break;
            case 6:
                cfg |= UART0_LCR_WLEN6;
                break;
            case 7:
                cfg |= UART0_LCR_WLEN7;
                break;
            case 8:
                cfg |= UART0_LCR_WLEN8;
                break;
            default:
                err = -1;
                break;
        }

        if (!err) {
            switch(parity) {
                case LPRINTF_UART_PARITY_NONE:
                    cfg |= UART0_LCR_PARITY_DIS;
                    break;
                case LPRINTF_UART_PARITY_ODD:
                    cfg |= UART0_LCR_PARITY_EN | UART0_LCR_PARITY_ODD;
                    break;
                case LPRINTF_UART_PARITY_EVEN:
                    cfg |= UART0_LCR_PARITY_EN | UART0_LCR_PARITY_EVEN;
                    break;
                default:
                    err = -1;
                    break;
            }
        }

        if (!err) {
            unsigned dl;
            unsigned fract_div;
            unsigned fract_mul;

            LPC_USART0->LCR = cfg;
            LPC_USART0->IER = 0;

            f_get_baudrate_settings(LPC_CLKF_USART0, baud, &dl, &fract_div, &fract_mul );

            if (fract_mul != 0) {
                LPC_USART0->LCR |= UART0_LCR_DLAB_EN;
                LPC_USART0->DLL = dl & 0xFF;
                LPC_USART0->DLM = (dl >> 8) & 0xFF;
                LPC_USART0->FDR =UART0_FDR_DIVADDVAL(fract_div) |
                             UART0_FDR_MULVAL(fract_mul);
                LPC_USART0->LCR &= ~UART0_LCR_DLAB_EN;
                LPC_USART0->FCR = UART0_FCR_FIFO_EN | UART0_FCR_RX_RS | UART0_FCR_TX_RS;;
                LPC_USART0->FCR = UART0_FCR_FIFO_EN;

                volatile unsigned long dummy __attribute__((__unused__));;
                dummy = LPC_USART0->IIR;
            } else {
                err = -1;
            }
        }
    }


    if (!err) {        
        LPC_PIN_CONFIG(tx_pin, 0);
        f_tx_pin = tx_pin;
        f_port = port;
    }
    return err;
}




void lprintf_putchar(char c) {
    static uint8_t rdy_size = UART_FIFO_SIZE;
    static uint8_t empty;

    do {
        empty = LPC_USART0->LSR & UART0_LSR_THRE;
    } while (!empty && (rdy_size==0));


    if (empty)
        rdy_size = UART_FIFO_SIZE;

    LPC_USART0->THR = c;
    rdy_size--;
}

void lprintf_close(void) {
    if (f_port==0) {
        /* ждем передачи всех данных */
        while (!(LPC_USART0->LSR & UART0_LSR_TEMT)) {}

        /* настройка пина в функцию по-умолчанию */
        LPC_PIN_CONFIG(LPC_PIN_ID(LPC_PIN_ID_PORT(f_tx_pin), LPC_PIN_ID_PIN(f_tx_pin), 0), 0);

        /* возвращаем значения по умолчанию */
        LPC_USART0->LCR = UART0_LCR_DLAB_EN;
        LPC_USART0->DLL = 1;
        LPC_USART0->DLM = 0;
        LPC_USART0->FDR = 0x10;
        LPC_USART0->LCR = 0;
        LPC_USART0->IER = 0;
        LPC_USART0->FCR = 0;


        LPC_CLK_DIS(SYSCTL_CLOCK_UART0);

    }
}
