#include "lprintf.h"

#include "LPC17xx.h"
#include "core_cm3.h"
#include "iolpc17XX.h"
#include "lpc17xx_pinfunc.h"
#include "lpc_uart_baudrate.inc"

#define UART_FIFO_SIZE 16

static LPC_UART1_TypeDef *f_uart;
static int f_uart_num;

static const struct {
    unsigned io_addr;
    unsigned irq;
    unsigned pconp_msk;
    unsigned char pclksel_pos;
    unsigned char pclksel_reg;
} f_uart_params[] = {
    { LPC_UART0_BASE, UART0_IRQn, LPC_SC_PCONP_PCUART0_Msk, LPC_SC_PCLKSEL0_PCLK_UART0_Pos, 0},
    { LPC_UART1_BASE, UART1_IRQn, LPC_SC_PCONP_PCUART1_Msk, LPC_SC_PCLKSEL0_PCLK_UART1_Pos, 0},
    { LPC_UART2_BASE, UART2_IRQn, LPC_SC_PCONP_PCUART2_Msk, LPC_SC_PCLKSEL1_PCLK_UART2_Pos, 1},
    { LPC_UART3_BASE, UART3_IRQn, LPC_SC_PCONP_PCUART3_Msk, LPC_SC_PCLKSEL1_PCLK_UART3_Pos, 1}
};

static const unsigned char clk_sel_div[] = {4, 1, 2, 8};

extern uint32_t SystemCoreClock;




/** @todo Сделать возможность выбора пина по tx_pin */
int lprintf_uart_init(unsigned port, unsigned baud, unsigned databits,
                      t_lprintf_uart_parity parity, unsigned tx_pin, unsigned flags) {
    unsigned long cfg = 0;
    int err = 0;

    if (port < sizeof(f_uart_params)/sizeof(f_uart_params[0])) {
        f_uart_num = port;
        f_uart = (LPC_UART1_TypeDef*) f_uart_params[port].io_addr;
        LPC_SC->PCONP |= f_uart_params[port].pconp_msk;


        if (port==0) {
            LPC_PINCON->PINSEL0 = (LPC_PINCON->PINSEL0 & ~(LPC_PINCON_PINSEL_PX2_Msk))
                    | LPC_PINFUNC_P0_2_TXD0;
        }
        /** @todo способ задания пинов для остальных портов */
    } else {
        err = -1;
    }



    if (!err) {
        switch (databits) {
            case 5:
                break;
            case 6:
                cfg |= 0x01 << LPC_UART_LCR_WordLengthSelect_Pos;
                break;
            case 7:
                cfg |= 0x02 << LPC_UART_LCR_WordLengthSelect_Pos;
                break;
            case 8:
                cfg |= 0x03 << LPC_UART_LCR_WordLengthSelect_Pos;
                break;
            default:
                err = -1;
        }
    }

    if (!err) {
        switch(parity) {
            case LPRINTF_UART_PARITY_NONE:
                break;
            case LPRINTF_UART_PARITY_ODD:
                cfg |= LPC_UART_LCR_ParityEnable_Msk;
                break;
            case LPRINTF_UART_PARITY_EVEN:
                cfg |= LPC_UART_LCR_ParityEnable_Msk | (0x01 << LPC_UART_LCR_ParitySelect_Pos);
                break;
            default:
                err = -1;
        }
    }

    if (!err) {
        f_uart->LCR = cfg;
        f_uart->IER = 0;


#if 0
        LPC_UART1->RS485CTRL = LPC_UART1_RS485CTRL_SEL_Msk | LPC_UART1_RS485CTRL_DCTRL_Msk;
#ifdef MB_PORT_RS485_DIR_INVERSE
        LPC_UART1->RS485CTRL |= LPC_UART1_RS485CTRL_OINV_Msk;
#endif
#endif

        unsigned dl;
        unsigned fract_div;
        unsigned fract_mul;

        unsigned uartclk = SystemCoreClock;
        unsigned char clk_sel = ((f_uart_params[port].pclksel_reg ? LPC_SC->PCLKSEL1 :
                                                                  LPC_SC->PCLKSEL0)
                >> f_uart_params[port].pclksel_pos) & 0x3;

        uartclk/=clk_sel_div[clk_sel];

        f_get_baudrate_settings(port, baud, &dl, &fract_div, &fract_mul );

        if (fract_mul != 0) {
            f_uart->LCR |= LPC_UART_LCR_DLAB_Msk;
            f_uart->DLL = dl & 0xFF;
            f_uart->DLM = (dl >> 8) & 0xFF;
            f_uart->FDR =((fract_div << LPC_UART_FDR_DIVADDVAL_Pos)
                    & LPC_UART_FDR_DIVADDVAL_Msk) |
                    ((fract_mul << LPC_UART_FDR_MULVAL_Pos) & LPC_UART_FDR_MULVAL_Msk);
            f_uart->LCR &= ~LPC_UART_LCR_DLAB_Msk;
            f_uart->FCR = LPC_UART_FCR_FIFOEnable_Msk | LPC_UART_FCR_RXFIFOReset_Msk |
                LPC_UART_FCR_TXFIFOReset_Msk;
            f_uart->FCR = LPC_UART_FCR_FIFOEnable_Msk;

            volatile unsigned long dummy __attribute__((__unused__));;
            dummy = f_uart->IIR;
        } else {
            err = -1;
        }
    }

    return err;
}

void lprintf_close(void) {
    /* ждем передачи всех данных */
    while (!(f_uart->LSR & LPC_UART_LSR_TEMT_Msk)) {}

    if (f_uart_num==0) {
        LPC_PINCON->PINSEL0 = (LPC_PINCON->PINSEL0 & ~(LPC_PINCON_PINSEL_PX2_Msk));
    }

    /* возвращаем значения по умолчанию */
    f_uart->LCR = LPC_UART_LCR_DLAB_Msk;
    f_uart->DLL = 1;
    f_uart->DLM = 0;
    f_uart->FDR = 0x10;
    f_uart->LCR = 0;
    f_uart->IER = 0;
    f_uart->FCR = 0;

    LPC_SC->PCONP &= ~f_uart_params[f_uart_num].pconp_msk;
}



void lprintf_putchar(char c) {
    static uint8_t rdy_size = UART_FIFO_SIZE;
    static uint8_t empty;

    do {
        empty = f_uart->LSR & LPC_UART_LSR_THRE_Msk;
    } while (!empty && (rdy_size==0));


    if (empty)
        rdy_size = UART_FIFO_SIZE;

    f_uart->THR = c;
    rdy_size--;
}
