#include "chip.h"
#include "lprintf.h"
#include "lpc_uart_baudrate.inc"

#define UART_FIFO_SIZE 16

static LPC_USART_T *f_uart;
static unsigned f_tx_pin;
static unsigned f_clk_src;
static unsigned f_base_clk;


int lprintf_uart_init(unsigned port, unsigned baud, unsigned databits,
                      t_lprintf_uart_parity parity, unsigned tx_pin, unsigned flags) {

    int err = 0;
    unsigned clk_cfg = 0;
    unsigned cfg = 0;

    switch (port) {
        case 0:
            f_uart = LPC_USART0;
            f_clk_src = CLK_MX_UART0;
            f_base_clk = CLK_BASE_UART0;
            clk_cfg = LPC_BASE_UART0_CFG;
            break;
        case 1:
            f_uart = LPC_UART1;
            f_clk_src = CLK_MX_UART1;
            f_base_clk = CLK_BASE_UART1;
            clk_cfg = LPC_BASE_UART1_CFG;
            break;
        case 2:
            f_uart = LPC_USART2;
            f_clk_src = CLK_MX_UART2;
            f_base_clk = CLK_BASE_UART2;
            clk_cfg = LPC_BASE_UART2_CFG;
            break;
        case 3:
            f_uart = LPC_USART3;
            f_clk_src = CLK_MX_UART3;
            f_base_clk = CLK_BASE_UART3;
            clk_cfg = LPC_BASE_UART3_CFG;
            break;
        default:
            err = -1;
            break;
    }

    if (!err) {
        switch (databits) {
            case 5:
                cfg |= UART_LCR_WLEN5;
                break;
            case 6:
                cfg |= UART_LCR_WLEN6;
                break;
            case 7:
                cfg |= UART_LCR_WLEN7;
                break;
            case 8:
                cfg |= UART_LCR_WLEN8;
                break;
            default:
                err = -1;
                break;
        }
    }

    if (!err) {
        switch(parity) {
            case LPRINTF_UART_PARITY_NONE:
                cfg |= UART_LCR_PARITY_DIS;
                break;
            case LPRINTF_UART_PARITY_ODD:
                cfg |= UART_LCR_PARITY_EN | UART_LCR_PARITY_ODD;
                break;
            case LPRINTF_UART_PARITY_EVEN:
                cfg |= UART_LCR_PARITY_EN | UART_LCR_PARITY_EVEN;
                break;
            default:
                err = -1;
                break;
        }
    }

    if (!err) {
        LPC_CGU->BASE_CLK[f_base_clk] = clk_cfg;
        lpc_delay_clk(20);
        LPC_CCU_CFG(f_clk_src,       1);
        lpc_delay_clk(20);

        f_uart->LCR = cfg;
        f_uart->IER = 0;


        unsigned dl;
        unsigned fract_div;
        unsigned fract_mul;
        f_get_baudrate_settings(LPC_CLKF_BASE(clk_cfg), baud, &dl, &fract_div, &fract_mul );

        if (fract_mul != 0) {
            f_uart->LCR |= UART_LCR_DLAB_EN;
            f_uart->DLL = dl & 0xFF;
            f_uart->DLM = (dl >> 8) & 0xFF;
            f_uart->FDR =UART_FDR_DIVADDVAL(fract_div) |
                         UART_FDR_MULVAL(fract_mul);
            f_uart->LCR &= ~UART_LCR_DLAB_EN;
            f_uart->FCR = UART_FCR_FIFO_EN | UART_FCR_RX_RS | UART_FCR_TX_RS;;
            f_uart->FCR = UART_FCR_FIFO_EN;

            volatile unsigned long dummy __attribute__((__unused__));;
            dummy = f_uart->IIR;
        } else {
            err = -1;
        }
    }

    if (!err && tx_pin) {
        CHIP_PIN_CONFIG(tx_pin, 0, CHIP_PIN_MODE_FAST | CHIP_PIN_MODE_NOPULL);

    }

    return err;
}




void lprintf_putchar(char c) {
    static uint8_t rdy_size = UART_FIFO_SIZE;
    static uint8_t empty;

    do {
        empty = f_uart->LSR & UART_LSR_THRE;
    } while (!empty && (rdy_size==0));


    if (empty)
        rdy_size = UART_FIFO_SIZE;

    f_uart->THR = c;
    rdy_size--;
}


void lprintf_close(void) {

    /* ждем передачи всех данных */
    while (!(f_uart->LSR & UART_LSR_TEMT)) {}

    /* настройка пина в функцию по-умолчанию */
    CHIP_PIN_CONFIG(CHIP_PIN_ID(CHIP_PIN_ID_p(f_tx_pin), CHIP_PIN_ID_n(f_tx_pin),
                              CHIP_PIN_ID_n(f_tx_pin), CHIP_PIN_ID_m(f_tx_pin),0), 0, 0);

    /* возвращаем значения по умолчанию */
    f_uart->LCR = UART_LCR_DLAB_EN;
    f_uart->DLL = 1;
    f_uart->DLM = 0;
    f_uart->FDR = 0x10;
    f_uart->LCR = 0;
    f_uart->IER = 0;
    f_uart->FCR = 0;



    LPC_CCU_CFG(f_clk_src,    0);
    LPC_CGU->BASE_CLK[f_base_clk] = 1;
}
