#include "lprintf.h"
#include <stdarg.h>

static inline void f_putstr_ch(char c) {
    static char prev_cr = 0;
    if ((c=='\n') && !prev_cr)
        lprintf_putchar('\r');
    prev_cr = c=='\r';
    lprintf_putchar(c);
}

static const char* f_atoi(const char* str, int *val) {
    int t_val = 0;
    const char *p = str;
    for (; (*str >= '0') && (*str <= '9'); str++) {
        t_val*=10;
        t_val+=*str-'0';
    }

    if (p!=str)
        *val = t_val;
    return str;
}

#define PRINT_FLG_NULL     0x01
#define PRINT_FLG_LJ       0x02
#define PRINT_FLG_PREF     0x04
#define PRINT_FLG_UPPER    0x08
#define PRINT_FLG_UNSIGNED 0x10

/**************************************************
 *  если с - флаг - обновляем flags и возвращаем 1
 *  иначе - возвращаем 0
 *************************************************/
static int f_check_flags(const char* c, int *flags) {
    int res = 0;
    switch (*c) {
    case '0':
        *flags |= PRINT_FLG_NULL;
        res = 1;
        break;
    case '-':
        *flags |= PRINT_FLG_LJ;
        res = 1;
        break;
    case '+':
        res = 1;
        break;
    case '#':
        *flags |=PRINT_FLG_PREF;
        res = 1;
        break;
    }
    return res;
}


#define PRINT_HEX_PREFIX() if (flags&PRINT_FLG_PREF) \
    { lprintf_putchar('0');    lprintf_putchar(flags & PRINT_FLG_UPPER ? 'X' : 'x'); }

/* печать в форматах %x и %X */
static void f_print_hex(int val, int flags, int width) {
    int i,j, start_print=0;


    for (i=2*sizeof(int)-1; i >= 0; i--) {
        char c = (val >> (i<<2)) & 0xF;
        if ((c || i==0) && !start_print) {
            start_print = i+1;
            /* выводим нужное кол-во нулей или пробелов перед */
            if (!(flags & PRINT_FLG_LJ)) {
                if (flags& PRINT_FLG_NULL) {
                    PRINT_HEX_PREFIX();
                }
                for (j=i+1; j < width; j++)
                    lprintf_putchar(flags & PRINT_FLG_NULL ? '0' : ' ');
                if (!(flags & PRINT_FLG_NULL)) {
                    PRINT_HEX_PREFIX();
                }
            } else {
                PRINT_HEX_PREFIX();
            }
        }

        /* печать символа из преобразования */
        if (c || start_print) {
            if (c<=9)
                lprintf_putchar('0'+c);
            else if (flags & PRINT_FLG_UPPER)
                lprintf_putchar('A' + c - 10);
            else
                lprintf_putchar('a' + c - 10);
        }
    }

    if (flags & PRINT_FLG_LJ) {
        for (j=start_print; j < width; j++)
            lprintf_putchar(' ');
    }
}

static void f_print_dec(int val, int flags, int width) {
    int digts=1, i, neg=0;
    unsigned int div, uval, max_div;

    if (flags & PRINT_FLG_UNSIGNED) {
        uval = (unsigned int)val;
    } else if (val < 0) {
        neg = 1;
        uval = -val;
    } else {
        uval = val;
    }

    max_div = (1UL << (sizeof(int)*8-1))/10;

    /* определяем кол-во цифр */
    for (digts=1, div=10; (uval >= div) && (div < max_div); digts++, div*=10)
    {}

    if (uval >= div) {
        digts++;
    } else {
        div/=10;
    }

    /* выводим начальные пробелы или нули */
    if (!(flags & PRINT_FLG_LJ)) {
        for (i = digts; i < width; i++)
            lprintf_putchar(flags & PRINT_FLG_NULL ? '0' : ' ');
    }

    if (neg)
        lprintf_putchar('-');


    while (div!=0) {
        char c;
        c = uval/div;
        uval %= div;
        lprintf_putchar('0' + c);
        div/=10;
    }

    /* если left just - выводим пробелы в конце, если нужны */
    if (flags & PRINT_FLG_LJ) {
        for (i = digts; i < width; i++)
            lprintf_putchar(' ');
    }
}

static void f_print_str(const char* str, int flags, int width) {
    int len = 0;
    if (!(flags & PRINT_FLG_LJ)) {
        const char *ptr;
        for (ptr = str; *ptr != '\0'; ptr++)
            len++;
        /* при обычном выравнивании выводим пробелы вначале, если длина
          строки меньше заданной ширины */
        for ( ; len < width; len++) {
            f_putstr_ch(' ');
        }
    }

    for (len=0; *str != '\0'; str++) {
        f_putstr_ch(*str);
        len++;
    }

    if (flags & PRINT_FLG_LJ) {
        for ( ; len < width; len++) {
            f_putstr_ch(' ');
        }
    }
}




int lprintf(const char* fmt, ...) {
    const char* first = fmt;
    va_list vl;
    va_start(vl,fmt);

    for (; *fmt!=0; fmt++)  {
        if (*fmt!='%') {
            f_putstr_ch(*fmt);
        } else {
            const char* start_fmt = fmt;
            fmt++;
            if (*fmt=='%') {
                lprintf_putchar('%');
            } else {
                int width = 0;
                int flags = 0;

                /* читаем флаги */
                while (f_check_flags(fmt, &flags))
                    fmt++;
                /* пробуем прочитать длину */
                fmt = f_atoi(fmt, &width);

                switch(*fmt) {
                    case 'X':{
                            unsigned val = va_arg(vl, unsigned);
                            flags|=PRINT_FLG_UPPER;
                            f_print_hex(val, flags, width);
                        }
                        break;
                    case 'x': {
                            unsigned val = va_arg(vl, unsigned);
                            f_print_hex(val, flags, width);
                        }
                        break;
                    case 'u': {
                            unsigned val = va_arg(vl, unsigned);
                            flags|=PRINT_FLG_UNSIGNED;
                            f_print_dec((int)val, flags, width);
                        }
                        break;
                    case 'd':{
                            int val = va_arg(vl, int);
                            f_print_dec(val, flags, width);
                        }
                        break;
                    case 's': {
                            const char* str = va_arg(vl, char*);
                            f_print_str(str, flags, width);
                        }
                        break;
                    case 'c': {
                            int ch = va_arg(vl, int);
                            f_putstr_ch((char)ch);
                        }
                        break;
                    default:
                        fmt = start_fmt;
                        break;
                }
            }
        }
    }

    va_end(vl);
    return (fmt-first);
}


