    # makefile для включения в проект.
#
# на входе принимает параметры:
#  TARGET_CPU (или может быть переопределен через LPRINTF_TARGET) - определяет,
#              какой порт будет использоваться. Должен быть задан один
#              из поддерживаемых портов
#  LPRINTF_DIR  - путь к данному файлу относительно директории сборки проекта
#             (по-умолчанию lib/lprintf)
#
# устанавливает следующие параметры:
#  LPRINTF_SRC      - исходные файлы на C
#  LPRINTF_INC_DIRS - директории с заголовками, которые должны быть добавлены
#                     в список директорий для поиска заголовков проекта


LPRINTF_DIR ?= ./lib/lprintf
LPRINTF_TARGET ?= $(TARGET_CPU)

LPRINTF_SRC := $(LPRINTF_DIR)/lprintf.c

LPRINTF_INC_DIRS := $(LPRINTF_DIR)


LPRINTF_PORT_SET = 0
#append port file to sources
ifeq ($(LPRINTF_TARGET), lpc17xx)
    LPRINTF_SRC += $(LPRINTF_DIR)/lpc/lpc17xx_uart.c
    LPRINTF_PORT_SET = 1
endif
ifeq ($(LPRINTF_TARGET), lpc43xx)
    LPRINTF_SRC += $(LPRINTF_DIR)/lpc/lpc43xx_uart.c
    LPRINTF_PORT_SET = 1
endif
ifeq ($(LPRINTF_TARGET), lpc11u6x)
    LPRINTF_SRC += $(LPRINTF_DIR)/lpc/lpc11u6x_uart.c
    LPRINTF_PORT_SET = 1
endif
ifeq ($(LPRINTF_TARGET), bf609)
    LPRINTF_SRC += $(LPRINTF_DIR)/bfin/bf609_uart.c
    LPRINTF_PORT_SET = 1
endif


ifneq ($(LPRINTF_PORT_SET), 1)
    $(error lprintf err: port architecture should be specified in LPRINTF_TARGET or TARGET_CPU variable)
endif
